@@ -19,9 +19,10 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ArrowButton.cc,v 1.5 2003/10/13 23:51:04 fluxgen Exp $
+// $Id: ArrowButton.cc 3700 2004-11-19 11:37:27Z mathias $
 
 #include "ArrowButton.hh"
+#include "ButtonTheme.hh"
 
 ArrowButton::ArrowButton(ArrowButton::Type arrow_type,
                          const FbTk::FbWindow &parent,
@@ -29,7 +30,8 @@ ArrowButton::ArrowButton(ArrowButton::Type arrow_type,
                          unsigned int width, unsigned int height):
     FbTk::Button(parent, x, y, width, height),
     m_arrow_type(arrow_type),
-    m_mouse_handler(0) {
+    m_mouse_handler(0),
+    m_arrowscale(300) {
 
     setEventMask(ExposureMask | ButtonPressMask | ButtonReleaseMask |
                  EnterWindowMask | LeaveWindowMask);
@@ -41,7 +43,8 @@ ArrowButton::ArrowButton(ArrowButton::Type arrow_type,
                          unsigned int width, unsigned int height):
     FbTk::Button(screen_num, x, y, width, height),
     m_arrow_type(arrow_type),
-    m_mouse_handler(0) {
+    m_mouse_handler(0),
+    m_arrowscale(300) {
 
     setEventMask(ExposureMask | ButtonPressMask | ButtonReleaseMask |
                  EnterWindowMask | LeaveWindowMask);
@@ -49,11 +52,13 @@ ArrowButton::ArrowButton(ArrowButton::Type arrow_type,
 
 void ArrowButton::clear() {
     FbTk::Button::clear();
+    updateTransparent();
     drawArrow();
 }
 
 void ArrowButton::exposeEvent(XExposeEvent &event) {
     FbTk::Button::exposeEvent(event);
+    updateTransparent(event.x, event.y, event.width, event.height);
     drawArrow();
 }
 
@@ -82,22 +87,37 @@ void ArrowButton::leaveNotifyEvent(XCrossingEvent &ce) {
 */
 void ArrowButton::drawArrow() {
     XPoint pts[3];
-    unsigned int w = width() / 2;
-    unsigned int h = height() / 2;
+    unsigned int w = width();
+    unsigned int h = height();
+
+    int arrowscale_n = m_arrowscale;
+    int arrowscale_d = 100;
+    unsigned int ax = arrowscale_d * w / arrowscale_n;
+    unsigned int ay = arrowscale_d * h / arrowscale_n;
+    // if these aren't an even number, left and right arrows end up different
+    if (( ax % 2 ) == 1) ax++;
+    if (( ay % 2 ) == 1) ay++;
     switch (m_arrow_type) {
     case LEFT:
-        pts[0].x = w - 2; pts[0].y = h;
-        pts[1].x = 4; pts[1].y = 2;
-        pts[2].x = 0; pts[2].y = -4;
+		// start at the tip
+        pts[0].x = (w / 2) - (ax / 2); pts[0].y = h / 2;
+        pts[1].x = ax; pts[1].y = -ay / 2;
+        pts[2].x = 0; pts[2].y = ay;
         break;
     case RIGHT:
-        pts[0].x = w - 2; pts[0].y = h - 2;
-        pts[1].x = 4; pts[1].y = 2;
-        pts[2].x = -4; pts[2].y = 2;
+        pts[0].x = (w / 2) + (ax / 2); pts[0].y = h / 2;
+        pts[1].x = - ax; pts[1].y = ay / 2;
+        pts[2].x = 0; pts[2].y = - ay;
         break;
-    case UP: // TODO
+    case UP:
+        pts[0].x = (w / 2); pts[0].y = (h / 2) - (ay / 2);
+        pts[1].x = ax / 2; pts[1].y = ay;
+        pts[2].x = - ax; pts[2].y = 0;
         break;
-    case DOWN: // TODO
+    case DOWN:
+        pts[0].x = (w / 2); pts[0].y = (h / 2) + (ay / 2);
+        pts[1].x = ax / 2; pts[1].y = - ay;
+        pts[2].x = - ax; pts[2].y = 0;
         break;
     }
 
@@ -108,3 +128,12 @@ void ArrowButton::drawArrow() {
     }
 }
 
+void ArrowButton::updateTheme(const FbTk::Theme &theme) {
+    // it must be a button theme
+    const ButtonTheme &btheme = static_cast<const ButtonTheme &>(theme);
+
+    m_arrowscale = btheme.scale();
+    if (m_arrowscale == 0) m_arrowscale = 300; // default is 0 => 300
+    else if (m_arrowscale < 100) m_arrowscale = 100; // otherwise clamp
+    else if (m_arrowscale > 100000) m_arrowscale = 100000; // clamp below overflow when *100
+}
