@@ -20,12 +20,13 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ClientPattern.cc,v 1.6 2003/12/17 01:19:39 fluxgen Exp $
+// $Id: ClientPattern.cc 3700 2004-11-19 11:37:27Z mathias $
 
 #include "ClientPattern.hh"
 #include "RegExp.hh"
 #include "StringUtil.hh"
 #include "WinClient.hh"
+#include "FbTk/App.hh"
 
 // use GNU extensions
 #ifndef _GNU_SOURCE
@@ -37,7 +38,11 @@
 #include <fstream>
 #include <string>
 #include <memory>
-#include <cstdio>
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
+#else
+  #include <stdio.h>
+#endif
 
 // needed as well for index on some systems (e.g. solaris)
 #include <strings.h> 
@@ -110,6 +115,8 @@ ClientPattern::ClientPattern(const char *str):
                     prop = CLASS;
                 } else if (strcasecmp(memstr.c_str(), "title") == 0) {
                     prop = TITLE;
+                } else if (strcasecmp(memstr.c_str(), "role") == 0) {
+                    prop = ROLE;
                 } else {
                     had_error = pos + match.find_first_of('(') + 1;
                     break;
@@ -186,6 +193,8 @@ std::string ClientPattern::toString() const {
         case TITLE:
             pat.append("title=");
             break;
+        case ROLE:
+            pat.append("role=");
         }
 
         pat.append((*it)->orig);
@@ -246,6 +255,10 @@ std::string ClientPattern::getProperty(WinProperty prop, const WinClient &client
     case NAME:
         return client.getWMClassName();
         break;
+    case ROLE:
+        Atom wm_role = XInternAtom(FbTk::App::instance()->display(), "WM_WINDOW_ROLE", False);
+        return client.textProperty(wm_role);
+        break;
     }
     return client.getWMClassName();
 }
