@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: CurrentWindowCmd.cc 3731 2004-11-30 01:23:24Z mathias $
+// $Id: CurrentWindowCmd.cc 3787 2004-12-21 16:09:36Z mathias $
 
 #include "CurrentWindowCmd.hh"
 
@@ -42,6 +42,10 @@ void KillWindowCmd::real_execute() {
     winclient().sendClose(true);
 }
 
+void SetHeadCmd::real_execute() {
+    fbwindow().screen().setOnHead(fbwindow(), m_head);
+}
+
 void SendToWorkspaceCmd::real_execute() {
     if (m_workspace_num >= 0 && m_workspace_num < fbwindow().screen().getNumberOfWorkspaces())
         fbwindow().screen().sendToWorkspace(m_workspace_num, &fbwindow());
@@ -60,6 +64,18 @@ void SendToPrevWorkspaceCmd::real_execute() {
     fbwindow().screen().sendToWorkspace(workspace_num, &fbwindow());
 }
 
+void GoToTabCmd::real_execute() {
+    if (fbwindow().numClients() <= 1 || m_tab_num > fbwindow().numClients())
+        return;
+
+    FluxboxWindow::ClientList::iterator it = fbwindow().clientList().begin();
+
+    int num = m_tab_num;
+    while (--num > 0) ++it;
+
+    fbwindow().setCurrentClient(**it, true);
+}
+
 void WindowHelperCmd::execute() {
     WinClient *client = Fluxbox::instance()->getFocusedWindow();
     if (client && client->fbwindow()) // guarantee that fbwindow() exists too
