@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbCommands.cc,v 1.24 2004/04/22 21:12:32 fluxgen Exp $
+// $Id: FbCommands.cc 3720 2004-11-24 12:26:12Z mathias $
 
 #include "FbCommands.hh"
 #include "fluxbox.hh"
@@ -37,8 +37,73 @@
 
 #include <fstream>
 #include <iostream>
+
+#ifdef HAVE_CSTDLIB
+  #include <cstdlib>
+#else
+  #include <stdlib.h>
+#endif
+
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
+#if defined(__EMX__) && defined(HAVE_PROCESS_H)
+#include <process.h> // for P_NOWAIT
+#endif // __EMX__
+
 using namespace std;
 
+namespace {
+
+void showMenu(const BScreen &screen, FbTk::Menu &menu) {
+
+    // special case for root menu
+    if (&menu == &screen.getRootmenu()) {
+        Fluxbox* fb = Fluxbox::instance();
+        if(fb->menuTimestampsChanged()) {
+            // we dont show the menu here because fluxbox
+            // will bring up the rootmenu after the timed
+            // reread of the menu
+            fb->rereadMenu(true);
+            return;
+        }
+    }
+    
+    Window root_ret; // not used
+    Window window_ret; // not used
+
+    int rx = 0, ry = 0;
+    int wx, wy; // not used
+    unsigned int mask; // not used
+
+    XQueryPointer(menu.fbwindow().display(),
+                  screen.rootWindow().window(), &root_ret, &window_ret,
+                  &rx, &ry, &wx, &wy, &mask);
+
+    int borderw = menu.fbwindow().borderWidth();
+    int head = screen.getHead(rx, ry);
+
+    pair<int, int> m = 
+        screen.clampToHead(head,
+                           rx - menu.width() / 2, 
+                           ry - menu.titleWindow().height() / 2,
+                           menu.width() + 2*borderw,
+                           menu.height() + 2*borderw);
+
+    menu.move(m.first, m.second);
+    menu.setScreen(screen.getHeadX(head),
+                   screen.getHeadY(head),
+                   screen.getHeadWidth(head),
+                   screen.getHeadHeight(head));
+    
+    menu.show();
+    menu.grabInputFocus();
+}
+
+}
+
 namespace FbCommands {
 
 ExecuteCmd::ExecuteCmd(const std::string &cmd, int screen_num):m_cmd(cmd), m_screen_num(screen_num) {
@@ -46,7 +111,7 @@ ExecuteCmd::ExecuteCmd(const std::string &cmd, int screen_num):m_cmd(cmd), m_scr
 }
 
 void ExecuteCmd::execute() {
-#ifndef    __EMX__
+#ifndef __EMX__
     if (! fork()) {
         std::string displaystring("DISPLAY=");
         displaystring += DisplayString(FbTk::App::instance()->display());
@@ -70,11 +135,25 @@ void ExecuteCmd::execute() {
         exit(0);
     }
 #else //   __EMX__
-    spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", item->exec().c_str(), 0);
+    spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", m_cmd.c_str(), 0);
 #endif // !__EMX__
 
 }
 
+ExportCmd::ExportCmd(const std::string& name, const std::string& value) :
+    m_name(name), m_value(value) { 
+}
+
+void ExportCmd::execute() {
+// TODO: we need to analyze this a bit more
+#if defined sgi && ! defined GCC
+    putenv((m_name + "=" + m_value).c_str());
+#else
+    setenv(m_name.c_str(), m_value.c_str(), 1);
+#endif
+}
+
+
 void ExitFluxboxCmd::execute() {
     Fluxbox::instance()->shutdown();
 }
@@ -118,25 +197,7 @@ void ShowRootMenuCmd::execute() {
     if (screen == 0)
         return;
 
-    Window root_ret;
-    Window window_ret;
-
-    int rx, ry;
-    int wx, wy;
-    unsigned int mask;
-
-    if ( XQueryPointer(FbTk::App::instance()->display(),
-                       screen->rootWindow().window(), &root_ret, &window_ret,
-                       &rx, &ry, &wx, &wy, &mask) ) {
-
-        if ( rx - (screen->getRootmenu().width()/2) > 0 )
-            rx-= screen->getRootmenu().width()/2;
-        screen->getRootmenu().move(rx, ry);
-    }
-
-    screen->getRootmenu().show();
-    screen->getRootmenu().grabInputFocus();
-
+    ::showMenu(*screen, screen->getRootmenu());
 }
 
 void ShowWorkspaceMenuCmd::execute() {
@@ -144,26 +205,7 @@ void ShowWorkspaceMenuCmd::execute() {
     if (screen == 0)
         return;
 
- 
-
-    Window root_ret;
-    Window window_ret;
-
-    int rx, ry;
-    int wx, wy;
-    unsigned int mask;
-
-    if ( XQueryPointer(FbTk::App::instance()->display(),
-                       screen->rootWindow().window(), &root_ret, &window_ret,
-                       &rx, &ry, &wx, &wy, &mask) ) {
-
-        if ( rx - (screen->getWorkspacemenu().width()/2) > 0 )
-            rx-= screen->getWorkspacemenu().width()/2;
-        screen->getWorkspacemenu().move(rx, ry);
-    }
-    screen->getWorkspacemenu().show();
-    screen->getWorkspacemenu().grabInputFocus();
- 
+    ::showMenu(*screen, screen->getWorkspacemenu());
 }
 
 
@@ -251,7 +293,7 @@ void BindKeyCmd::execute() {
 }
 
 DeiconifyCmd::DeiconifyCmd(const Mode mode, 
-    const Destination dest) : m_mode(mode), m_dest(dest) { }
+                           const Destination dest) : m_mode(mode), m_dest(dest) { }
 
 void DeiconifyCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
@@ -267,35 +309,35 @@ void DeiconifyCmd::execute() {
 
     switch(m_mode) {
         
-        case ALL:
-        case ALLWORKSPACE:
-            for(; it != itend; it++) {
-                old_workspace_num= (*it)->workspaceNumber();
-                if (m_mode == ALL || old_workspace_num == workspace_num) {
-                    if (m_dest == ORIGIN || m_dest == ORIGINQUIET)
-                        screen->sendToWorkspace(old_workspace_num, (*it), change_ws);
-                    else
-                        (*it)->deiconify(false);
-                }
+    case ALL:
+    case ALLWORKSPACE:
+        for(; it != itend; it++) {
+            old_workspace_num= (*it)->workspaceNumber();
+            if (m_mode == ALL || old_workspace_num == workspace_num) {
+                if (m_dest == ORIGIN || m_dest == ORIGINQUIET)
+                    screen->sendToWorkspace(old_workspace_num, (*it), change_ws);
+                else
+                    (*it)->deiconify(false);
             }
-            break;
+        }
+        break;
 
-        case LAST:
-        case LASTWORKSPACE:
-        default:
-            for (; it != itend; it++) {
-                old_workspace_num= (*it)->workspaceNumber();
-                if(m_mode == LAST || old_workspace_num == workspace_num) {
-                    if ((m_dest == ORIGIN || m_dest == ORIGINQUIET) && 
-                        m_mode != LASTWORKSPACE)
-                        screen->sendToWorkspace(old_workspace_num, (*it), change_ws);
-                    else
-                        (*it)->deiconify(false);
-                    break;
-                }
+    case LAST:
+    case LASTWORKSPACE:
+    default:
+        for (; it != itend; it++) {
+            old_workspace_num= (*it)->workspaceNumber();
+            if(m_mode == LAST || old_workspace_num == workspace_num) {
+                if ((m_dest == ORIGIN || m_dest == ORIGINQUIET) && 
+                    m_mode != LASTWORKSPACE)
+                    screen->sendToWorkspace(old_workspace_num, (*it), change_ws);
+                else
+                    (*it)->deiconify(false);
+                break;
             }
-            break;
-   };
+        }
+        break;
+    };
 }
     
 }; // end namespace FbCommands
