@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbMenu.cc,v 1.3 2004/04/18 18:53:55 fluxgen Exp $
+// $Id: FbMenu.cc 3770 2004-12-13 14:03:17Z mathias $
 
 
 #include "FbMenu.hh"
@@ -39,8 +39,8 @@ FbMenu::~FbMenu() {
 
 }
 
-void FbMenu::update(int index) {
-    FbTk::Menu::update(index);
+void FbMenu::updateMenu(int index) {
+    FbTk::Menu::updateMenu(index);
     m_shape->update();
 }
 
