@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-//$Id: Font.cc 3701 2004-11-19 12:00:20Z mathias $
+//$Id: Font.cc 3793 2004-12-21 23:42:09Z mathias $
 
 
 #include "StringUtil.hh"
@@ -229,9 +229,14 @@ namespace FbTk {
 bool Font::m_multibyte = false; 
 bool Font::m_utf8mode = false;
 
-// some initialisation for using fonts
-void fontInit() {
-   setlocale(LC_CTYPE, "");
+
+void Font::init() {
+    // must be set before the first XFontSet is created
+    setlocale(LC_CTYPE, "");
+}
+
+void Font::shutdown() {
+
 }
 
 Font::Font(const char *name, bool antialias):
