@@ -1,5 +1,5 @@
 // GContext.cc for FbTk - fluxbox toolkit
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003-2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: GContext.cc,v 1.6 2004/01/09 01:19:48 fluxgen Exp $
+// $Id: GContext.cc 3832 2005-01-12 06:22:26Z mathias $
 
 #include "GContext.hh"
 
@@ -34,11 +34,10 @@ namespace FbTk {
 Display *GContext::m_display = 0;
 
 GContext::GContext(const FbTk::FbDrawable &drawable): 
-    m_gc(XCreateGC(m_display != 0 ? m_display : FbTk::App::instance()->display(),
-                   drawable.drawable(),
-                   0, 0)) {
+    m_gc(XCreateGC(drawable.display(), drawable.drawable(), 0, 0)) {
+    
     if (m_display == 0)
-        m_display = FbTk::App::instance()->display();
+        m_display = drawable.display();
 
     setGraphicsExposure(false);
 }
