@@ -1,5 +1,5 @@
 // MenuTheme.cc for FbTk
-// Copyright (c) 2002-2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002-2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: MenuTheme.cc,v 1.14 2004/01/08 22:07:58 fluxgen Exp $
+// $Id: MenuTheme.cc 3700 2004-11-19 11:37:27Z mathias $
 
 #include "MenuTheme.hh"
 
@@ -29,7 +29,12 @@
 #include "App.hh"
 #include "StringUtil.hh"
 
-#include <cstdio>
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
+#else
+  #include <stdio.h>
+#endif
+#include <algorithm>
 
 namespace FbTk {
 
@@ -48,6 +53,8 @@ MenuTheme::MenuTheme(int screen_num):
     titlefont_justify(*this, "menu.title.justify", "Menu.Title.Justify"),
     bullet_pos(*this, "menu.bullet.position", "Menu.Bullet.Position"),
     m_bullet(*this, "menu.bullet", "Menu.Bullet"),
+    m_title_height(*this, "menu.titleHeight", "Menu.TitleHeight"),
+    m_item_height(*this, "menu.itemHeight", "Menu.ItemHeight"),
     m_border_width(*this, "menu.borderWidth", "Menu.BorderWidth"),
     m_bevel_width(*this, "menu.bevelWidth", "Menu.BevelWidth"),
     m_border_color(*this, "menu.borderColor", "Menu.BorderColor"),
@@ -63,8 +70,16 @@ MenuTheme::MenuTheme(int screen_num):
     m_alpha(255),
     m_menumode(DELAY_OPEN),
     m_delayopen(0), // no delay as default
-    m_delayclose(0) // no delay as default
+    m_delayclose(0), // no delay as default
+    m_real_title_height(*m_title_height),
+    m_real_item_height(*m_item_height)
 { 
+    m_real_item_height = std::max(*m_item_height, frameFont().height() + 2*bevelWidth());
+    m_real_title_height = std::max(*m_title_height,
+                                   titleFont().height() + 2*bevelWidth());
+
+    m_real_item_height = m_real_item_height == 0 ? 1 : m_real_item_height;
+    m_real_title_height = m_real_title_height == 0 ? 1 : m_real_title_height;
 
     // set default values
     *m_border_width = 0;
@@ -90,9 +105,16 @@ void MenuTheme::reconfigTheme() {
     if (*m_border_width > 20)
         *m_border_width = 20;
 
-    m_bullet_pixmap->scale(frameFont().height(), frameFont().height());
-    m_selected_pixmap->scale(frameFont().height(), frameFont().height());
-    m_unselected_pixmap->scale(frameFont().height(), frameFont().height());
+    m_real_item_height = std::max(*m_item_height, frameFont().height() + 2*bevelWidth());
+    m_real_title_height = std::max(*m_title_height,
+                                   titleFont().height() + 2*bevelWidth());
+
+    m_real_item_height = m_real_item_height == 0 ? 1 : m_real_item_height;
+    m_real_title_height = m_real_title_height == 0 ? 1 : m_real_title_height;
+
+    m_bullet_pixmap->scale(itemHeight(), itemHeight());
+    m_selected_pixmap->scale(itemHeight(), itemHeight());
+    m_unselected_pixmap->scale(itemHeight(), itemHeight());
 
     t_text_gc.setForeground(*t_text);
     f_text_gc.setForeground(*f_text);
@@ -136,7 +158,7 @@ void ThemeItem<MenuTheme::BulletType>::setFromString(const char *str) {
 }
 
 template <>
-void ThemeItem<MenuTheme::BulletType>::load() {
+void ThemeItem<MenuTheme::BulletType>::load(const std::string *name, const std::string *altname) {
     // do nothing, we don't have anything extra to load
 }
 
@@ -151,7 +173,7 @@ void ThemeItem<unsigned int>::setFromString(const char *str) {
 }
 
 template <>
-void ThemeItem<unsigned int>::load() {
+void ThemeItem<unsigned int>::load(const std::string *name, const std::string *altname) {
 }
 
 
