@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: TextBox.cc,v 1.9 2004/04/19 18:57:30 fluxgen Exp $
+// $Id: TextBox.cc 3784 2004-12-18 02:44:11Z mathias $
 
 #include "TextBox.hh"
 #include "Font.hh"
@@ -27,7 +27,11 @@
 #include "App.hh"
 #include "KeyUtil.hh"
 
-#include <cctype>
+#ifdef HAVE_CCTYPE
+  #include <cctype>
+#else
+  #include <ctype.h>
+#endif
 #include <X11/keysym.h>
 #include <X11/Xutil.h>
 
@@ -65,6 +69,7 @@ TextBox::~TextBox() {
 
 void TextBox::setText(const std::string &text) {
     m_text = text;
+    m_start_pos = 0;
     cursorEnd();
     adjustStartPos();
 }
@@ -86,10 +91,13 @@ void TextBox::setInputFocus() {
 
 void TextBox::cursorHome() {
     m_start_pos = m_cursor_pos = 0;
+    adjustEndPos();
 }
 
 void TextBox::cursorEnd() {
-    m_cursor_pos = m_end_pos = text().size();
+    m_end_pos = text().size();
+    adjustStartPos();
+    m_cursor_pos = m_end_pos - m_start_pos;
 }
 
 void TextBox::cursorForward() {
@@ -133,6 +141,7 @@ void TextBox::insertText(const std::string &val) {
     m_text.insert(m_start_pos + cursorPosition(), val);
     m_cursor_pos += val.size();
     m_end_pos += val.size();
+   
     if (m_start_pos + cursorPosition() < m_end_pos)
         adjustEndPos();
     else
@@ -153,7 +162,7 @@ void TextBox::clear() {
     if (gc() == 0)
         setGC(DefaultGC(FbTk::App::instance()->display(), screenNumber()));
 
-    font().drawText(window(), screenNumber(), 
+    font().drawText(*this, screenNumber(), 
                     gc(),                     
                     text().c_str() + m_start_pos, 
                     m_end_pos - m_start_pos, 
@@ -181,6 +190,22 @@ void TextBox::exposeEvent(XExposeEvent &event) {
 
 void TextBox::buttonPressEvent(XButtonEvent &event) {
     setInputFocus();
+    if (event.window == window()) {
+        std::string::size_type i;
+        std::string::size_type click_pos = m_end_pos;
+        int delta = width();
+        int tmp = 0;
+        for(i = m_start_pos; i <= m_end_pos; i++) {
+            tmp = abs(event.x - font().textWidth(m_text.c_str() + m_start_pos, i - m_start_pos));
+
+            if (tmp < delta) {
+                delta = tmp;
+                click_pos = i;
+            }
+        }
+        m_cursor_pos = click_pos - m_start_pos;
+        clear();
+    }
 }
 
 void TextBox::keyPressEvent(XKeyEvent &event) {
@@ -193,7 +218,7 @@ void TextBox::keyPressEvent(XKeyEvent &event) {
     // a modifier key by itself doesn't do anything
     if (IsModifierKey(ks)) return;
 
-    if (event.state) { // handle keybindings without state
+    if (event.state) { // handle keybindings with state
         if (event.state == ControlMask) {
 
             switch (ks) {
@@ -218,11 +243,12 @@ void TextBox::keyPressEvent(XKeyEvent &event) {
             case XK_c:
                 cursorHome();
                 m_text = "";
-                adjustStartPos();
-                adjustEndPos();
+                m_start_pos = 0;
+                m_cursor_pos = 0;
+                m_end_pos = 0;
                 break;
             }
-        } else if (event.state == ShiftMask || event.state == 0x2000) {
+        } else if (event.state == ShiftMask || event.state == 0x80) { // shif and altgr
             if (isprint(keychar[0])) {
                 std::string val;
                 val += keychar[0];
@@ -248,7 +274,45 @@ void TextBox::keyPressEvent(XKeyEvent &event) {
         case XK_Right:
             cursorForward();
             break;
+        case XK_Delete:
+            deleteForward();
+            break;
         default:
+            switch (ks) {
+            case XK_KP_Insert:
+                keychar[0] = '0';
+                break;
+            case XK_KP_End:
+                keychar[0] = '1';
+                break;
+            case XK_KP_Down:
+                keychar[0] = '2';
+                break;
+            case XK_KP_Page_Down:
+                keychar[0] = '3';
+                break;
+            case XK_KP_Left:
+                keychar[0] = '4';
+                break;
+            case XK_KP_Begin:
+                keychar[0] = '5';
+                break;
+            case XK_KP_Right:
+                keychar[0] = '6';
+                break;
+            case XK_KP_Home:
+                keychar[0] = '7';
+                break;
+            case XK_KP_Up:
+                keychar[0] = '8';
+                break;
+            case XK_KP_Page_Up:
+                keychar[0] = '9';
+                break;
+            case XK_KP_Delete:
+                keychar[0] = ',';
+                break;
+            };
             if (isprint(keychar[0])) {
                 std::string val;
                 val += keychar[0];
@@ -275,16 +339,17 @@ void TextBox::adjustEndPos() {
 }
 
 void TextBox::adjustStartPos() {
-    int text_width = font().textWidth(text().c_str() + m_start_pos, m_end_pos - m_start_pos);
+    
+    int text_width = font().textWidth(text().c_str(), m_end_pos);
     if (text_width < static_cast<signed>(width()))
         return;
 
     int start_pos = 0;
-    text_width = font().textWidth(text().c_str() + start_pos, m_end_pos - start_pos);
     while (text_width > static_cast<signed>(width())) {
         start_pos++;
         text_width = font().textWidth(text().c_str() + start_pos, m_end_pos - start_pos);
     }
+
     // adjust cursorPosition() according relative to change to m_start_pos
     m_cursor_pos -= start_pos - m_start_pos;
     m_start_pos = start_pos;
