@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: TextureRender.cc,v 1.8 2004/01/21 20:19:50 fluxgen Exp $
+// $Id: TextureRender.cc 3827 2005-01-11 12:39:01Z mathias $
 
 #include "TextureRender.hh"
 
@@ -30,12 +30,24 @@
 #include "App.hh"
 #include "FbPixmap.hh"
 #include "GContext.hh"
+#include "I18n.hh"
 
 #include <iostream>
 #include <string>
-#include <cstdio>
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
+#else
+  #include <stdio.h>
+#endif
 using namespace std;
 
+// mipspro has no new(nothrow)
+#if defined sgi && ! defined GCC
+#define FB_new_nothrow new
+#else
+#define FB_new_nothrow new(std::nothrow)
+#endif
+
 namespace FbTk {
 
 TextureRender::TextureRender(ImageControl &imgctrl, 
@@ -44,45 +56,24 @@ TextureRender::TextureRender(ImageControl &imgctrl,
     control(imgctrl),
     colors(_colors),
     ncolors(ncolors),
+    cpc(imgctrl.colorsPerChannel()),
+    cpccpc(cpc * cpc),
+    red(0), green(0), blue(0),
+    width(static_cast<signed>((w > 0 ? w : 1))), height(static_cast<signed>(h > 0 ? h : 1)),
     xtable(0), ytable(0) {
 
-    width = ((signed) w > 0) ? w : 1;
-    height = ((signed) h > 0) ? h : 1;
+    _FB_USES_NLS;
     // clamp to "normal" size
     if (width > 3200) {
-        cerr<<"TextureRender: Warning! Width > 3200 setting Width = 3200"<<endl;
+        cerr<<"TextureRender: "<<_FBTKTEXT(Error, BigWidth, "Warning! Width > 3200 setting Width = 3200", "Image width seems too big, clamping")<<endl;
         width = 3200;
     }
 
     if (height > 3200) {
-        cerr<<"TextureRender: Warning! Height > 3200 setting Height = 3200"<<endl;
+        cerr<<"TextureRender: "<<_FBTKTEXT(Error, BigHeight, "Warning! Height > 3200 setting Height = 3200", "Image height seems too big, clamping")<<endl;
         height = 3200;
     }
 
-    red = new(nothrow) unsigned char[width * height];
-    if (red == 0) {
-        char sbuf[128];
-        sprintf(sbuf, "%d", width*height);
-        throw string("TextureRender::TextureRender(): Out of memory while allocating red buffer."+ string(sbuf));
-    }
-
-
-    green = new(nothrow) unsigned char[width * height];
-    if (green == 0) {
-        char sbuf[128];
-        sprintf(sbuf, "%d", width*height);
-        throw string("TextureRender::TextureRender(): Out of memory while allocating green buffer. size " + string(sbuf));
-    }
-
-    blue = new(nothrow) unsigned char[width * height];
-    if (blue == 0) {
-        char sbuf[128];
-        sprintf(sbuf, "%d", width*height);
-        throw string("TextureRender::TextureRender(): Out of memory while allocating blue buffer. size " + string(sbuf));
-    }
-
-    cpc = imgctrl.colorsPerChannel();
-    cpccpc = cpc * cpc;
 
     imgctrl.colorTables(&red_table, &green_table, &blue_table,
                         &red_offset, &green_offset, &blue_offset,
@@ -92,9 +83,9 @@ TextureRender::TextureRender(ImageControl &imgctrl,
 
 
 TextureRender::~TextureRender() {
-    delete [] red;
-    delete [] green;
-    delete [] blue;	
+    if (red != 0) delete [] red;
+    if (green != 0) delete [] green;
+    if (blue != 0) delete [] blue;	
 }
 
 
@@ -105,12 +96,47 @@ Pixmap TextureRender::render(const FbTk::Texture &texture) {
         return ParentRelative;
     else if (texture.type() & FbTk::Texture::SOLID)
         return renderSolid(texture);
-    else if (texture.type() & FbTk::Texture::GRADIENT)
+    else if (texture.type() & FbTk::Texture::GRADIENT) {
+        allocateColorTables();
         return renderGradient(texture);
+    }
 
     return None;
 }
 
+void TextureRender::allocateColorTables() {
+
+    _FB_USES_NLS;
+
+    const size_t size = width * height;
+    red = FB_new_nothrow unsigned char[size];
+
+    if (red == 0) {
+        char sbuf[128];
+        sprintf(sbuf, "%ld", (long int) size);
+        throw std::string("TextureRender::TextureRender(): " + 
+              std::string(_FBTKTEXT(Error, OutOfMemoryRed, "Out of memory while allocating red buffer.", "")) + string(sbuf));
+    }
+
+
+    green = FB_new_nothrow unsigned char[size];
+    if (green == 0) {
+        char sbuf[128];
+        sprintf(sbuf, "%ld", (long int) size);
+        throw std::string("TextureRender::TextureRender(): " + 
+              std::string(_FBTKTEXT(Error, OutOfMemoryGreen, "Out of memory while allocating green buffer.", ""))+ string(sbuf));
+    }
+
+    blue = FB_new_nothrow unsigned char[size];
+    if (blue == 0) {
+        char sbuf[128];
+        sprintf(sbuf, "%ld", (long int) size);
+        throw std::string("TextureRender::TextureRender(): " +
+              std::string(_FBTKTEXT(Error, OutOfMemoryBlue, "Out of memory while allocating blue buffer.", ""))+ string(sbuf));
+    }
+
+
+}
 
 Pixmap TextureRender::renderSolid(const FbTk::Texture &texture) {
 
@@ -120,7 +146,8 @@ Pixmap TextureRender::renderSolid(const FbTk::Texture &texture) {
                     control.depth());
 
     if (pixmap.drawable() == None) {
-        cerr<<"FbTk::TextureRender::render_solid(): error creating pixmap"<<endl;
+        _FB_USES_NLS;
+        cerr<<"FbTk::TextureRender::render_solid(): "<<_FBTKTEXT(Error, CreatePixmap, "Error creating pixmap", "Couldn't create a pixmap - image - for some reason")<<endl;
         return None;
     }
 
@@ -253,12 +280,16 @@ Pixmap TextureRender::renderGradient(const FbTk::Texture &texture) {
 Pixmap TextureRender::renderPixmap(const FbTk::Texture &src_texture) {
     if (width != src_texture.pixmap().width() ||
         height != src_texture.pixmap().height()) {
+
         // copy src_texture's pixmap and 
-        // scale to fit our size
+        // scale/tile to fit our size
         FbPixmap new_pm(src_texture.pixmap());
-        // if not tiled then scale it
-        if (! (src_texture.type() & Texture::TILED))
+ 
+        if ((src_texture.type() & Texture::TILED)) {
+            new_pm.tile(width,height);
+        } else {
             new_pm.scale(width, height);
+        }
 
         return new_pm.release();
     }
@@ -274,7 +305,8 @@ XImage *TextureRender::renderXImage() {
                      width, height, 32, 0);
 
     if (! image) {
-        cerr<<"FbTk::TextureRender::renderXImage(): error creating XImage"<<endl;
+        _FB_USES_NLS;
+        cerr<<"FbTk::TextureRender::renderXImage(): "<<_FBTKTEXT(Error, CreateXImage, "Error creating XImage", "Couldn't create an XImage")<<endl;
         return 0;
     }
 
@@ -288,7 +320,7 @@ XImage *TextureRender::renderXImage() {
 
     o = image->bits_per_pixel + ((image->byte_order == MSBFirst) ? 1 : 0);
 
-    if (control.doDither() && width > 1 && height > 1) {
+    if (control.doDither()) {
         unsigned char dither4[4][4] = { 
             {0, 4, 1, 5},
             {6, 2, 7, 3},
@@ -545,12 +577,16 @@ XImage *TextureRender::renderXImage() {
         */
 
     default:
-        cerr<<"TextureRender::renderXImage(): unsupported visual"<<endl;
+        _FB_USES_NLS;
+        cerr<<"TextureRender::renderXImage(): "<<
+            _FBTKTEXT(Error, UnsupportedVisual, "unsupported visual", "A visual is a technical term in X")<<endl;
         delete [] d;
         XDestroyImage(image);
         return (XImage *) 0;
     }
-} else {
+} else { // end do dither
+
+    // no dither:
     switch (control.visual()->c_class) {
     case StaticColor:
     case PseudoColor:
@@ -644,7 +680,9 @@ XImage *TextureRender::renderXImage() {
 	break;
 
     default:
-        cerr<<"TextureRender::renderXImage(): unsupported visual"<<endl;
+        _FB_USES_NLS;
+        cerr<<"TextureRender::renderXImage(): "<<
+            _FBTKTEXT(Error, UnsupportedVisual, "unsupported visual", "A visual is a technical term in X")<<endl;
         delete [] d;
         XDestroyImage(image);
         return (XImage *) 0;
@@ -662,7 +700,8 @@ Pixmap TextureRender::renderPixmap() {
                     width, height, control.depth());
 
     if (pixmap.drawable() == None) {
-        cerr<<"TextureRender::renderPixmap(): error creating pixmap"<<endl;
+        _FB_USES_NLS;
+        cerr<<"FbTk::TextureRender::renderPixmap(): "<<_FBTKTEXT(Error, CreatePixmap, "Error creating pixmap", "Couldn't create a pixmap - image - for some reason")<<endl;
         return None;
     }
 
