@@ -20,22 +20,24 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: IconButton.cc,v 1.18 2004/02/18 10:16:23 fluxgen Exp $
+// $Id: IconButton.cc 3813 2005-01-05 01:56:19Z mathias $
 
 #include "IconButton.hh"
-
+#include "IconbarTool.hh"
 
 #include "fluxbox.hh"
 #include "Screen.hh"
 #include "Window.hh"
 #include "WinClient.hh"
+#include "CommandParser.hh"
 
-#include "FbTk/SimpleCommand.hh"
 #include "FbTk/App.hh"
+#include "FbTk/SimpleCommand.hh"
 #include "FbTk/EventManager.hh"
 #include "FbTk/MacroCommand.hh"
 #include "FbTk/Command.hh"
 #include "FbTk/RefCount.hh"
+#include "FbTk/Menu.hh"
 
 
 #ifdef HAVE_CONFIG_H
@@ -47,6 +49,8 @@
 #include <X11/extensions/shape.h>
 #endif // SHAPE
 
+typedef FbTk::RefCount<FbTk::Command> RefCmd;
+
 namespace {
 
 class ShowMenu: public FbTk::Command {
@@ -54,11 +58,11 @@ public:
     explicit ShowMenu(FluxboxWindow &win):m_win(win) { }
     void execute() {
         m_win.screen().hideMenus();
+        m_win.menu().enableTitle();
         // get last button pos
         const XEvent &event = Fluxbox::instance()->lastEvent();
         int x = event.xbutton.x_root - (m_win.menu().width() / 2);
         int y = event.xbutton.y_root - (m_win.menu().height() / 2);
-
         m_win.showMenu(x, y);
     }
 private:
@@ -67,51 +71,105 @@ private:
 
 class FocusCommand: public FbTk::Command {
 public:
-    explicit FocusCommand(FluxboxWindow &win):m_win(win) { }
+    explicit FocusCommand(const IconbarTool& tool, FluxboxWindow &win) : 
+        m_win(win), m_tool(tool) { }
     void execute() {
         if(m_win.isIconic() || !m_win.isFocused()) {
-            m_win.screen().changeWorkspaceID(m_win.workspaceNumber());
+            switch(m_tool.deiconifyMode()) {
+            case IconbarTool::SEMIFOLLOW:
+                if (m_win.isIconic()) {
+                    m_win.screen().sendToWorkspace(m_win.screen().currentWorkspaceID(), &m_win);
+                } else {
+                    m_win.screen().changeWorkspaceID(m_win.workspaceNumber());
+                }
+                break;
+            case IconbarTool::CURRENT:
+                m_win.screen().sendToWorkspace(m_win.screen().currentWorkspaceID(), &m_win);
+                break;
+            case IconbarTool::FOLLOW:
+            default:
+                m_win.screen().changeWorkspaceID(m_win.workspaceNumber());
+                break;
+            };
             m_win.raiseAndFocus();
        } else
            m_win.iconify();
     }
+
 private:
     FluxboxWindow &m_win;
+    const IconbarTool& m_tool;
 };
 
+// simple forwarding of wheeling, but only 
+// if desktopwheeling is enabled
+class WheelWorkspaceCmd : public FbTk::Command {
+public:
+    explicit WheelWorkspaceCmd(const IconbarTool& tool, FluxboxWindow &win, const char* cmd) : 
+        m_win(win), m_tool(tool), m_cmd(CommandParser::instance().parseLine(cmd)){ }
+    void execute() {
+
+        switch(m_tool.wheelMode()) {
+        case IconbarTool::ON:
+            m_cmd->execute();
+            break;
+        case IconbarTool::SCREEN:
+            if(m_win.screen().isDesktopWheeling())
+                m_cmd->execute();
+            break;
+        case IconbarTool::OFF:
+        default:
+            break;
+        };
+    }
+
+private:
+    FluxboxWindow &m_win;
+    RefCmd m_cmd;
+    const IconbarTool& m_tool;
+};
 
 } // end anonymous namespace
 
 
 
-IconButton::IconButton(const FbTk::FbWindow &parent, const FbTk::Font &font,
-                       FluxboxWindow &win):
+IconButton::IconButton(const IconbarTool& tool, const FbTk::FbWindow &parent, 
+                       const FbTk::Font &font, FluxboxWindow &win):
     FbTk::TextButton(parent, font, win.winClient().title()),
     m_win(win), 
     m_icon_window(*this, 1, 1, 1, 1, 
                   ExposureMask | ButtonPressMask | ButtonReleaseMask),
     m_use_pixmap(true) {
 
-    FbTk::RefCount<FbTk::Command> hidemenus(new FbTk::SimpleCommand<BScreen>(win.screen(), &BScreen::hideMenus));
+
+    RefCmd next_workspace(new ::WheelWorkspaceCmd(tool, m_win, "nextworkspace"));
+    RefCmd prev_workspace(new ::WheelWorkspaceCmd(tool, m_win, "prevworkspace"));
+
     //!! TODO: There're some issues with MacroCommand when
     //         this object dies when the last macrocommand is executed (focused cmd)
     //         In iconbar mode Icons
+    //
+    //    RefCmd hidemenus(new FbTk::SimpleCommand<BScreen>(win.screen(), &BScreen::hideMenus));
     //    FbTk::MacroCommand *focus_macro = new FbTk::MacroCommand();
     //    focus_macro->add(hidemenus);
     //    focus_macro->add(focus);
-    FbTk::RefCount<FbTk::Command> focus_cmd(new ::FocusCommand(m_win));
-    FbTk::RefCount<FbTk::Command> menu_cmd(new ::ShowMenu(m_win));
+    
+    RefCmd focus_cmd(new ::FocusCommand(tool, m_win));
+    RefCmd menu_cmd(new ::ShowMenu(m_win));
     setOnClick(focus_cmd, 1);
     setOnClick(menu_cmd, 3);
+    setOnClick(next_workspace, 4);
+    setOnClick(prev_workspace, 5);
+   
     m_win.hintSig().attach(this);
     
     FbTk::EventManager::instance()->add(*this, m_icon_window);
-    
+
     update(0);
 }
 
 IconButton::~IconButton() {
-
+    // ~FbWindow cleans event manager
 }
 
 
@@ -121,21 +179,24 @@ void IconButton::exposeEvent(XExposeEvent &event) {
     else
         FbTk::TextButton::exposeEvent(event);
 }
+
 void IconButton::moveResize(int x, int y,
                             unsigned int width, unsigned int height) {
 
     FbTk::TextButton::moveResize(x, y, width, height);
 
     if (m_icon_window.width() != FbTk::Button::width() ||
-        m_icon_window.height() != FbTk::Button::height())
+        m_icon_window.height() != FbTk::Button::height()) {
         update(0); // update icon window
+    }
 }
 
 void IconButton::resize(unsigned int width, unsigned int height) {
     FbTk::TextButton::resize(width, height);
     if (m_icon_window.width() != FbTk::Button::width() ||
-        m_icon_window.height() != FbTk::Button::height())
+        m_icon_window.height() != FbTk::Button::height()) {
         update(0); // update icon window
+    }
 }
 
 void IconButton::clear() {
@@ -209,7 +270,11 @@ void IconButton::update(FbTk::Subject *subj) {
 
 #endif // SHAPE
 
-    setupWindow();
+    if (subj != 0) {
+        setupWindow();
+    } else {
+        m_icon_window.clear();
+    }
 }
 
 void IconButton::setupWindow() {
