@@ -1,5 +1,5 @@
 // IconbarTool.cc
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003-2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: IconbarTool.cc,v 1.35 2004/03/22 20:08:08 fluxgen Exp $
+// $Id: IconbarTool.cc 3770 2004-12-13 14:03:17Z mathias $
 
 #include "IconbarTool.hh"
 
@@ -35,12 +35,14 @@
 #include "CommandParser.hh"
 #include "WinClient.hh"
 
+#include "FbTk/I18n.hh"
 #include "FbTk/Menu.hh"
 #include "FbTk/MenuItem.hh"
 #include "FbTk/RefCount.hh"
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/ImageControl.hh"
 #include "FbTk/MacroCommand.hh"
+#include "FbTk/MenuSeparator.hh"
 
 #include <typeinfo>
 #include <string>
@@ -63,6 +65,68 @@ void FbTk::Resource<IconbarTool::Mode>::setFromString(const char *strval) {
         setDefaultValue();
 }
 
+template<>
+void FbTk::Resource<IconbarTool::DeiconifyMode>::setDefaultValue() {
+    m_value = IconbarTool::FOLLOW;
+}
+
+template<>
+void FbTk::Resource<IconbarTool::DeiconifyMode>::setFromString(const char* strval) {
+    if (strncasecmp(strval, "Current", strlen("Current")) == 0)
+        m_value = IconbarTool::CURRENT;
+    else if (strncasecmp(strval, "Follow", strlen("Follow")) == 0)
+        m_value = IconbarTool::FOLLOW;
+    else if (strncasecmp(strval, "SemiFollow", strlen("SemiFollow")) == 0)
+        m_value = IconbarTool::SEMIFOLLOW;
+    else
+        setDefaultValue();
+}
+
+template<>
+std::string FbTk::Resource<IconbarTool::DeiconifyMode>::getString() {
+    switch (m_value) {
+    case IconbarTool::SEMIFOLLOW:
+        return std::string("SemiFollow");
+        break;
+    case IconbarTool::CURRENT:
+        return std::string("Current");
+        break;
+    };
+    return std::string("Follow");
+}
+
+template<>
+void FbTk::Resource<IconbarTool::WheelMode>::setDefaultValue() {
+    m_value = IconbarTool::SCREEN;
+}
+
+
+template<>
+void FbTk::Resource<IconbarTool::WheelMode>::setFromString(const char* strval) {
+    if (strncasecmp(strval, "off", strlen("off")) == 0)
+        m_value = IconbarTool::OFF;
+    else if (strncasecmp(strval, "on", strlen("on")) == 0)
+        m_value = IconbarTool::ON;
+    else if (strncasecmp(strval, "screen", strlen("screen")) == 0)
+        m_value = IconbarTool::SCREEN;
+    else
+        setDefaultValue();
+}
+
+
+template<>
+std::string FbTk::Resource<IconbarTool::WheelMode>::getString() {
+    switch(m_value) {
+    case IconbarTool::ON:
+        return std::string("On");
+        break;
+    case IconbarTool::SCREEN:
+        return std::string("Screen");
+        break;
+    };
+    return std::string("Off");
+}
+
 template<>
 void FbTk::Resource<Container::Alignment>::setDefaultValue() {
     m_value = Container::RELATIVE;
@@ -87,7 +151,7 @@ void FbTk::Resource<Container::Alignment>::setFromString(const char *str) {
         m_value = Container::LEFT;
     else if (strcasecmp(str, "Right") == 0)
         m_value = Container::RIGHT;
-    else if (strcasecmp(str, "RELATIVE") == 0)
+    else if (strcasecmp(str, "Relative") == 0)
         m_value = Container::RELATIVE;
     else
         setDefaultValue();
@@ -157,34 +221,63 @@ private:
 
 void setupModeMenu(FbTk::Menu &menu, IconbarTool &handler) {
     using namespace FbTk;
+    _FB_USES_NLS;
 
-    // TODO: nls
-    menu.setLabel("Iconbar Mode");
+    menu.setLabel(_FBTEXT(Toolbar, IconbarMode, "Iconbar Mode", "Menu title - chooses which set of icons are shown in the iconbar"));
 
     RefCount<Command> saverc_cmd(new SimpleCommand<Fluxbox>(
                                                             *Fluxbox::instance(), 
                                                             &Fluxbox::save_rc));
     
-    //TODO: nls
-    menu.insert(new ToolbarModeMenuItem("None", handler, 
-                                        IconbarTool::NONE, saverc_cmd));
-    menu.insert(new ToolbarModeMenuItem("Icons", handler, 
-                                        IconbarTool::ICONS, saverc_cmd));
-    menu.insert(new ToolbarModeMenuItem("Workspace Icons", handler, 
-                                        IconbarTool::WORKSPACEICONS, saverc_cmd));
-    menu.insert(new ToolbarModeMenuItem("Workspace", handler, 
-                                        IconbarTool::WORKSPACE, saverc_cmd));
-    menu.insert(new ToolbarModeMenuItem("All Windows", handler, 
-                                        IconbarTool::ALLWINDOWS, saverc_cmd));
-    menu.insert("---"); // separator line
-    menu.insert(new ToolbarAlignMenuItem("Left", handler,
-                                         Container::LEFT, saverc_cmd));
-    menu.insert(new ToolbarAlignMenuItem("Relative", handler,
-                                         Container::RELATIVE, saverc_cmd));
-    menu.insert(new ToolbarAlignMenuItem("Right", handler,
-                                         Container::RIGHT, saverc_cmd));
-    menu.insert("---"); // separator line
-    menu.update();
+
+    menu.insert(new ToolbarModeMenuItem(_FBTEXT(Toolbar, IconbarModeNone, 
+                                                "None", "No icons are shown in the iconbar"),
+                    handler, 
+                    IconbarTool::NONE, saverc_cmd));
+
+    menu.insert(new ToolbarModeMenuItem(
+                    _FBTEXT(Toolbar, IconbarModeIcons, 
+                            "Icons", "Iconified windows from all workspaces are shown"),
+                    handler, 
+                    IconbarTool::ICONS, saverc_cmd));
+
+    menu.insert(new ToolbarModeMenuItem(
+                    _FBTEXT(Toolbar, IconbarModeWorkspaceIcons, 
+                            "WorkspaceIcons", "Iconified windows from this workspace are shown"),
+                    handler,
+                    IconbarTool::WORKSPACEICONS, saverc_cmd));
+
+    menu.insert(new ToolbarModeMenuItem(
+                    _FBTEXT(Toolbar, IconbarModeWorkspace, 
+                            "Workspace", "Normal and iconified windows from this workspace are shown"),
+                    handler, 
+                    IconbarTool::WORKSPACE, saverc_cmd));
+
+    menu.insert(new ToolbarModeMenuItem(
+                    _FBTEXT(Toolbar, IconbarModeAllWindows, "All Windows", "All windows are shown"),
+                    handler, 
+                    IconbarTool::ALLWINDOWS, saverc_cmd));
+
+    menu.insert(new FbTk::MenuSeparator());
+
+    menu.insert(new ToolbarAlignMenuItem(
+                    _FBTEXT(Align, Left, "Left", "Align to the left"),
+                    handler,
+                    Container::LEFT, saverc_cmd));
+
+    menu.insert(new ToolbarAlignMenuItem(
+                    _FBTEXT(Align, Relative, "Relative", "Align relative to the width"),
+                    handler,
+                    Container::RELATIVE, saverc_cmd));
+
+    menu.insert(new ToolbarAlignMenuItem(
+                    _FBTEXT(Align, Right, "Right", "Align to the right"),
+                    handler,
+                    Container::RIGHT, saverc_cmd));
+
+    menu.insert(new FbTk::MenuSeparator());
+
+    menu.updateMenu();
 }
                 
 inline bool checkAddWindow(IconbarTool::Mode mode, const FluxboxWindow &win) {
@@ -226,7 +319,7 @@ void removeDuplicate(const IconbarTool::IconList &iconlist, std::list<FluxboxWin
 
     // remove already existing windows
     windowlist.erase(remove_it, windowlist.end());
-
+    windowlist.unique();
 }
 
 }; // end anonymous namespace
@@ -244,10 +337,18 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme, BScr
     m_empty_pm(0),
     m_rc_mode(screen.resourceManager(), WORKSPACE,
               screen.name() + ".iconbar.mode", screen.altName() + ".Iconbar.Mode"),
+    m_deiconify_mode(screen.resourceManager(), FOLLOW,
+                     screen.name() + ".iconbar.deiconifyMode",
+                     screen.name() + ".iconbar.DeiconifyMode"),
+    m_wheel_mode(screen.resourceManager(), OFF,
+                 screen.name() + ".iconbar.wheelMode",
+                 screen.name() + ".iconbar.WheelMode"),
     m_rc_alignment(screen.resourceManager(), Container::LEFT,
                    screen.name() + ".iconbar.alignment", screen.altName() + ".Iconbar.Alignment"),
     m_rc_client_width(screen.resourceManager(), 70,
-                   screen.name() + ".iconbar.clientWidth", screen.altName() + ".Iconbar.ClientWidth"),
+                   screen.name() + ".iconbar.iconWidth", screen.altName() + ".Iconbar.IconWidth"),
+    m_rc_client_padding(screen.resourceManager(), 10,
+                   screen.name() + ".iconbar.iconTextPadding", screen.altName() + ".Iconbar.IconTextPadding"),
     m_rc_use_pixmap(screen.resourceManager(), true,
                     screen.name() + ".iconbar.usePixmap", screen.altName() + ".Iconbar.UsePixmap"),
     m_menu(screen.menuTheme(), screen.imageControl(),
@@ -255,7 +356,7 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme, BScr
 
     // setup mode menu
     setupModeMenu(m_menu, *this);
-
+    _FB_USES_NLS;
     using namespace FbTk;
     // setup use pixmap item to reconfig iconbar and save resource on click
     MacroCommand *save_and_reconfig = new MacroCommand();   
@@ -264,8 +365,9 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme, BScr
     save_and_reconfig->add(reconfig);
     save_and_reconfig->add(save);
     RefCount<Command> s_and_reconfig(save_and_reconfig);
-    m_menu.insert(new BoolMenuItem("Show Pictures", *m_rc_use_pixmap, s_and_reconfig));
-    m_menu.update();
+    m_menu.insert(new BoolMenuItem(_FBTEXT(Toolbar, ShowIcons, "Show Pictures", "chooses if little icons are shown next to title in the iconbar") , 
+	                *m_rc_use_pixmap, s_and_reconfig));
+    m_menu.updateMenu();
     // must be internal menu, otherwise toolbar main menu tries to delete it.
     m_menu.setInternalMenu();
 
@@ -278,6 +380,7 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme, BScr
     screen.iconListSig().attach(this);
     screen.currentWorkspaceSig().attach(this);
     // setup focus timer
+
     FbTk::RefCount<FbTk::Command> timer_cmd(new FbTk::SimpleCommand<IconbarTool>(*this, &IconbarTool::timedRender));
     timeval to;
     to.tv_sec = 0;
@@ -369,6 +472,20 @@ void IconbarTool::setMode(Mode mode) {
     renderTheme();
 }
 
+void IconbarTool::setDeiconifyMode(DeiconifyMode mode) {
+    if (mode == *m_deiconify_mode)
+        return;
+
+    *m_deiconify_mode = mode;
+}
+
+void IconbarTool::setWheelMode(WheelMode mode) {
+    if (mode == *m_wheel_mode)
+        return;
+    *m_wheel_mode = mode;
+}
+
+
 unsigned int IconbarTool::width() const {
     return m_icon_container.width();
 }
@@ -405,7 +522,6 @@ void IconbarTool::update(FbTk::Subject *subj) {
     if (mode() == NONE) {
         if (subj != 0 && typeid(*subj) == typeid(IconbarTheme))
             renderTheme();
-                
         return;
     }
 
@@ -417,7 +533,6 @@ void IconbarTool::update(FbTk::Subject *subj) {
             // start focus timer, so we can update without flicker
             m_focus_timer.start();
 
-            //renderWindow(winsubj->win());
             return;
         } else if (subj == &(winsubj->win().workspaceSig())) {
             // we can ignore this signal if we're in ALLWINDOWS mode
@@ -459,10 +574,7 @@ void IconbarTool::update(FbTk::Subject *subj) {
         if (&m_screen.currentWorkspaceSig() == screen_subj &&
             mode() != ALLWINDOWS && mode() != ICONS) {
             remove_all = true; // remove and readd all windows
-        }/* else if (&m_screen.iconListSig() == screen_subj &&
-            (mode() == ALLWINDOWS || mode() == ICONS || mode() == WORKSPACE)) {
-            remove_all = true;
-            }*/
+        }
     }
 
     // lock graphic update
@@ -493,7 +605,9 @@ void IconbarTool::update(FbTk::Subject *subj) {
     m_icon_container.update();
     m_icon_container.showSubwindows();
     
-    renderTheme();
+    // another renderTheme we hopefully shouldn't need? These renders
+    // should be done individually above
+    // renderTheme();
 }
 
 IconButton *IconbarTool::findButton(FluxboxWindow &win) {
@@ -515,10 +629,9 @@ void IconbarTool::renderWindow(FluxboxWindow &win) {
     renderButton(*button);
 }
 
+void IconbarTool::updateSizing() {
+    m_icon_container.setBorderWidth(m_theme.border().width());
 
-void IconbarTool::renderTheme() {
-
-    // update button sizes before we get max width per client!
     IconList::iterator icon_it = m_icon_list.begin();
     const IconList::iterator icon_it_end = m_icon_list.end(); 
     for (; icon_it != icon_it_end; ++icon_it) {
@@ -527,6 +640,19 @@ void IconbarTool::renderTheme() {
         else // unfocused
             (*icon_it)->setBorderWidth(m_theme.unfocusedBorder().width());
     }    
+
+}
+
+void IconbarTool::renderTheme(unsigned char alpha) {
+
+    m_alpha = alpha;
+    renderTheme();
+}
+
+void IconbarTool::renderTheme() {
+
+    // update button sizes before we get max width per client!
+    updateSizing();
     
     Pixmap tmp = m_focused_pm;
     Pixmap err_tmp = m_focused_err_pm;
@@ -582,27 +708,32 @@ void IconbarTool::renderTheme() {
     if (tmp)
         m_screen.imageControl().removeImage(tmp);
 
+    // set to zero so its consistent and not ugly
     m_icon_container.setBorderWidth(m_theme.border().width());
     m_icon_container.setBorderColor(m_theme.border().color());
-    m_icon_container.setAlpha(m_theme.alpha());
+    m_icon_container.setAlpha(m_alpha);
 
     // update buttons
-    icon_it = m_icon_list.begin();
-    for (; icon_it != icon_it_end; ++icon_it)
+    IconList::iterator icon_it = m_icon_list.begin();
+    const IconList::iterator icon_it_end = m_icon_list.end(); 
+    for (; icon_it != icon_it_end; ++icon_it) {
         renderButton(*(*icon_it));
+    }
 }
 
-void IconbarTool::renderButton(IconButton &button) {
+void IconbarTool::renderButton(IconButton &button, bool clear) {
 
     button.setPixmap(*m_rc_use_pixmap);
-    button.setAlpha(m_theme.alpha());
+    button.setAlpha(m_alpha);
 
-    // if we're rendering a button, there must be a back button.
     // The last button is always the regular width
-    bool wider_button = (button.width() != m_icon_container.back()->width());
+    bool wider_button = false;
+    if (!m_icon_container.empty())
+        wider_button = (button.width() != m_icon_container.back()->width());
 
     if (button.win().isFocused()) { // focused texture
         m_icon_container.setSelected(m_icon_container.find(&button));
+
         button.setGC(m_theme.focusedText().textGC());     
         button.setFont(m_theme.focusedText().font());
         button.setJustify(m_theme.focusedText().justify());
@@ -614,9 +745,7 @@ void IconbarTool::renderButton(IconButton &button) {
         else if (wider_button && m_focused_err_pm != 0)
             button.setBackgroundPixmap(m_focused_err_pm);
         else
-            button.setBackgroundColor(m_theme.focusedTexture().color());            
-
-
+            button.setBackgroundColor(m_theme.focusedTexture().color());        
 
     } else { // unfocused
         if (m_icon_container.selected() == &button)
@@ -634,11 +763,10 @@ void IconbarTool::renderButton(IconButton &button) {
             button.setBackgroundPixmap(m_unfocused_err_pm);
         else
             button.setBackgroundColor(m_theme.unfocusedTexture().color());
-
     }
 
-    button.clear();
-    button.updateTransparent();
+    if (clear)
+        button.clear(); // the clear also updates transparent
 }
 
 void IconbarTool::deleteIcons() {
@@ -658,9 +786,12 @@ void IconbarTool::removeWindow(FluxboxWindow &win) {
             break;
     }
     // did we find it?
-    if (it == m_icon_list.end())
+    if (it == m_icon_list.end()) {
         return;
-    
+    }
+#ifdef DEBUG
+    cerr<<"IconbarTool::"<<__FUNCTION__<<"( 0x"<<&win<<" title = "<<win.title()<<") found!"<<endl;
+#endif // DEBUG    
     // detach from all signals
     win.focusSig().detach(this);
     win.dieSig().detach(this);
@@ -683,8 +814,18 @@ void IconbarTool::addWindow(FluxboxWindow &win) {
     // we just want windows that has clients
     if (win.clientList().empty() || win.isIconHidden() )
         return;
+#ifdef DEBUG
+    cerr<<"IconbarTool::addWindow(0x"<<&win<<" title = "<<win.title()<<")"<<endl;
+#endif // DEBUG
+    IconButton *button = new IconButton(*this, 
+                                        m_icon_container, 
+                                        m_theme.focusedText().font(), 
+                                        win);
+  
 
-    IconButton *button = new IconButton(m_icon_container, m_theme.focusedText().font(), win);
+    button->setTextPadding(*m_rc_client_padding);
+
+    renderButton(*button, false); // update the attributes, but don't clear it
     m_icon_container.insertItem(button);    
     m_icon_list.push_back(button);
 
@@ -694,6 +835,7 @@ void IconbarTool::addWindow(FluxboxWindow &win) {
     win.workspaceSig().attach(this);
     win.stateSig().attach(this);
     win.titleSig().attach(this);
+
 }
 
 void IconbarTool::updateIcons() {
@@ -759,6 +901,7 @@ void IconbarTool::addList(std::list<FluxboxWindow *> &winlist) {
     std::list<FluxboxWindow *>::iterator it_end = winlist.end();
     for (; it != it_end; ++it)
         addWindow(**it);
+    renderTheme();
 }
 
 void IconbarTool::timedRender() {
@@ -776,6 +919,5 @@ void IconbarTool::timedRender() {
         renderButton(*button);
     if (current_button != 0)
         renderButton(*current_button);
-
 }
 
