@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Slit.cc,v 1.93 2004/04/19 22:42:05 fluxgen Exp $
+// $Id: Slit.cc 3770 2004-12-13 14:03:17Z mathias $
 
 #include "Slit.hh"
 
@@ -35,7 +35,6 @@
 #include "config.h"
 #endif // HAVE_CONFIG_H
 
-#include "I18n.hh"
 #include "Screen.hh"
 #include "ImageControl.hh"
 #include "RefCount.hh"
@@ -57,11 +56,17 @@
 #include "SlitClient.hh"
 #include "Xutil.hh"
 #include "FbAtoms.hh"
+#include "FbTk/MenuSeparator.hh"
 #include "FbTk/StringUtil.hh"
+#include "FbTk/I18n.hh"
 
 #include <algorithm>
 #include <iostream>
-#include <cassert>
+#ifdef HAVE_CASSERT
+  #include <cassert>
+#else
+  #include <assert.h>
+#endif
 
 #ifdef HAVE_SYS_STAT_H
 #include <sys/types.h>
@@ -79,8 +84,9 @@ template<>
 void FbTk::Resource<Slit::Placement>::setFromString(const char *strval) {
     if (strcasecmp(strval, "TopLeft")==0)
         m_value = Slit::TOPLEFT;
-    else if (strcasecmp(strval, "CenterLeft")==0)
-        m_value = Slit::CENTERLEFT;
+    else if (strcasecmp(strval, "LeftCenter")==0
+             || strcasecmp(strval, "CenterLeft")==0)
+        m_value = Slit::LEFTCENTER;
     else if (strcasecmp(strval, "BottomLeft")==0)
         m_value = Slit::BOTTOMLEFT;
     else if (strcasecmp(strval, "TopCenter")==0)
@@ -89,8 +95,9 @@ void FbTk::Resource<Slit::Placement>::setFromString(const char *strval) {
         m_value = Slit::BOTTOMCENTER;
     else if (strcasecmp(strval, "TopRight")==0)
         m_value = Slit::TOPRIGHT;
-    else if (strcasecmp(strval, "CenterRight")==0)
-        m_value = Slit::CENTERRIGHT;
+    else if (strcasecmp(strval, "RightCenter")==0
+             || strcasecmp(strval, "CenterRight")==0)
+        m_value = Slit::RIGHTCENTER;
     else if (strcasecmp(strval, "BottomRight")==0)
         m_value = Slit::BOTTOMRIGHT;
     else
@@ -113,8 +120,8 @@ string FbTk::Resource<Slit::Placement>::getString() {
     case Slit::TOPLEFT:
         return string("TopLeft");
         break;
-    case Slit::CENTERLEFT:
-        return string("CenterLeft");
+    case Slit::LEFTCENTER:
+        return string("LeftCenter");
         break;
     case Slit::BOTTOMLEFT:
         return string("BottomLeft");
@@ -128,8 +135,8 @@ string FbTk::Resource<Slit::Placement>::getString() {
     case Slit::TOPRIGHT:
         return string("TopRight");
         break;
-    case Slit::CENTERRIGHT:
-        return string("CenterRight");
+    case Slit::RIGHTCENTER:
+        return string("RightCenter");
         break;
     case Slit::BOTTOMRIGHT:
         return string("BottomRight");
@@ -155,14 +162,13 @@ string FbTk::Resource<Slit::Direction>::getString() {
 
 namespace { 
 
-class SlitClientMenuItem: public FbTk::MenuItem {
+class SlitClientMenuItem: public FbTk::MenuItem{
 public:
-    explicit SlitClientMenuItem(SlitClient &client, FbTk::RefCount<FbTk::Command> &cmd):
-        FbTk::MenuItem(client.matchName().c_str(), cmd), m_client(client) {
+    explicit SlitClientMenuItem(Slit& slit, SlitClient &client, FbTk::RefCount<FbTk::Command> &cmd):
+        FbTk::MenuItem(client.matchName().c_str(), cmd), m_slit(slit), m_client(client) {
+        setCommand(cmd);
         FbTk::MenuItem::setSelected(client.visible());
-        // save resources as default click action
-        FbTk::RefCount<FbTk::Command> save_rc(new FbCommands::SaveResources());
-        setCommand(save_rc);
+        setToggleItem(true);
     }
     const std::string &label() const {
         return m_client.matchName();
@@ -170,11 +176,19 @@ public:
     bool isSelected() const {
         return m_client.visible();
     }
-    void click(int button, int time) { 
-        m_client.setVisible(!m_client.visible());
-        FbTk::MenuItem::click(button, time);
+    void click(int button, int time) {
+        if (button == 4 || button == 2) { // wheel up
+            m_slit.clientUp(&m_client);
+        } else if (button == 5 || button == 3) { // wheel down
+            m_slit.clientDown(&m_client);
+        } else {
+            m_client.setVisible(!m_client.visible());
+            FbTk::MenuItem::setSelected(m_client.visible());
+            FbTk::MenuItem::click(button, time);
+        }
     }
 private:
+    Slit& m_slit;
     SlitClient &m_client;
 };
 
@@ -198,14 +212,13 @@ public:
     }
 
     void setLabel(const char *label) {
-        I18n *i18n = I18n::instance();
+        _FB_USES_NLS;
         m_label = (label ? label : "");
         std::string reallabel = m_label + " " + 
             ( m_slit.direction() == Slit::HORIZONTAL ? 
-              i18n->getMessage(FBNLS::CommonSet, FBNLS::CommonDirectionHoriz,
-                               "Horizontal") :
-              i18n->getMessage(FBNLS::CommonSet, FBNLS::CommonDirectionVert,
-                               "Vertical") );
+              
+              _FBTEXT(Align, Horizontal, "Horizontal", "Horizontal"):
+              _FBTEXT(Align, Vertical,   "Vertical",   "Vertical"));
         FbTk::MenuItem::setLabel(reallabel.c_str());
     }
 private:
@@ -246,12 +259,8 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
       m_clientlist_menu(scr.menuTheme(),
                         scr.imageControl(),
                         *scr.layerManager().getLayer(Fluxbox::instance()->getMenuLayer())),
-      m_layermenu(new LayerMenu<Slit>(scr.menuTheme(),
-                                      scr.imageControl(),
-                                      *scr.layerManager().getLayer(Fluxbox::instance()->getMenuLayer()), 
-                                      this,
-                                      true)),
-      //For KDE dock applets
+      frame(scr.rootWindow()),
+       //For KDE dock applets
       m_kwm1_dockwindow(XInternAtom(FbTk::App::instance()->display(), 
                                     "KWM_DOCKWINDOW", False)), //KDE v1.x
       m_kwm2_dockwindow(XInternAtom(FbTk::App::instance()->display(), 
@@ -278,6 +287,8 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
       m_rc_layernum(scr.resourceManager(), Fluxbox::Layer(Fluxbox::instance()->getDockLayer()), 
                     scr.name() + ".slit.layer", scr.altName() + ".Slit.Layer") {
 
+    _FB_USES_NLS;
+
     // attach to theme and root window change signal
     m_slit_theme->reconfigSig().attach(this);
     scr.resizeSig().attach(this);
@@ -290,36 +301,26 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
     FbTk::RefCount<FbTk::Command> toggle_hidden(new FbTk::SimpleCommand<Slit>(*this, &Slit::toggleHidden));
     m_timer.setCommand(toggle_hidden);
 
-    // create main window
-    XSetWindowAttributes attrib;
-    unsigned long create_mask = CWBackPixmap | CWBackPixel | CWBorderPixel |
-        CWColormap | CWOverrideRedirect | CWEventMask;
-    attrib.background_pixmap = None;
-    attrib.background_pixel = attrib.border_pixel = theme().borderColor().pixel();
-    attrib.colormap = screen().rootWindow().colormap();
-    attrib.override_redirect = True;
-    attrib.event_mask = s_eventmask;
-
-    frame.x = frame.y = 0;
-    frame.width = frame.height = 1;
-    Display *disp = FbTk::App::instance()->display();
-    frame.window =
-        XCreateWindow(disp, screen().rootWindow().window(), frame.x, frame.y,
-                      frame.width, frame.height, theme().borderWidth(),
-                      screen().rootWindow().depth(), InputOutput, screen().rootWindow().visual(),
-                      create_mask, &attrib);
 
     FbTk::EventManager::instance()->add(*this, frame.window);
-    m_transp.reset(new FbTk::Transparent(screen().rootPixmap(), frame.window.drawable(), 
-                                         *m_rc_alpha,
-                                         screen().screenNumber()));
+    
+    if (FbTk::Transparent::haveComposite()) {
+        frame.window.setOpaque(*m_rc_alpha);
+    } else {
+        frame.window.setAlpha(*m_rc_alpha);
+    }
 
     m_layeritem.reset(new FbTk::XLayerItem(frame.window, layer));
+    m_layermenu.reset(new LayerMenu<Slit>(scr.menuTheme(),
+                                          scr.imageControl(),
+                                          *scr.layerManager().
+                                          getLayer(Fluxbox::instance()->getMenuLayer()), 
+                                          this,
+                                          true));
     moveToLayer((*m_rc_layernum).getNum());
 
-    // TODO: nls
     if (m_layermenu.get())
-        m_layermenu->setLabel("Slit Layer");
+        m_layermenu->setLabel(_FBTEXT(Slit, Layer, "Slit Layer", "Title of Slit Layer Menu"));
 
     // Get client list for sorting purposes
     loadClientList(filename);
@@ -393,16 +394,16 @@ void Slit::updateStrut() {
         else
             right = width();
         break;
-    case CENTERLEFT:
+    case LEFTCENTER:
         if (direction() == VERTICAL)
             left = width();        
         break;
-    case CENTERRIGHT:
+    case RIGHTCENTER:
         if (direction() == VERTICAL)
             right = width();
         break;
     }
-    m_strut = screen().requestStrut(left, right, top, bottom);
+    m_strut = screen().requestStrut(getOnHead(), left, right, top, bottom);
     screen().updateAvailableWorkspaceArea();
 }
 
@@ -480,8 +481,10 @@ void Slit::addClient(Window w) {
         }
         
         XFree((void *) proto);
+#ifdef DEBUG
     } else {
         cerr<<"Warning: Failed to read WM Protocols. "<<endl;
+#endif // DEBUG
     }
 
     XWindowAttributes attrib;
@@ -689,15 +692,11 @@ void Slit::reconfigure() {
     if (tmp) 
         image_ctrl.removeImage(tmp);
 
-    if (m_transp.get()) {
-        if (frame.pixmap == 0)
-            m_transp->setDest(frame.window.drawable(), frame.window.screenNumber());
-        else
-            m_transp->setDest(frame.pixmap, frame.window.screenNumber());
-
-        m_transp->setAlpha(*m_rc_alpha);
+    if (FbTk::Transparent::haveComposite()) {
+        frame.window.setOpaque(*m_rc_alpha);
+    } else {
+        frame.window.setAlpha(*m_rc_alpha);
     }
-
     clearWindow();
 
     int x = 0, y = 0;
@@ -814,7 +813,7 @@ void Slit::reposition() {
         }
         break;
 
-    case CENTERLEFT:
+    case LEFTCENTER:
         frame.x = head_x;
         frame.y = head_y + (head_h - frame.height) / 2;
         frame.x_hidden = head_x + bevel_width -
@@ -866,7 +865,7 @@ void Slit::reposition() {
         }
         break;
 
-    case CENTERRIGHT:
+    case RIGHTCENTER:
     default:
         frame.x = head_x + head_w - frame.width - border_width*2;
         frame.y = head_y + ((head_h - frame.height) / 2);
@@ -907,6 +906,48 @@ void Slit::shutdown() {
         removeClient(m_client_list.front(), true, true);
 }
 
+void Slit::clientUp(SlitClient* client) {
+    if (!client || m_client_list.size() < 2)
+        return;
+    
+    if (client == m_client_list.front()) {
+        cycleClientsUp();
+        return;
+    }
+    
+    SlitClients::iterator it = m_client_list.begin();
+    for(it++; it != m_client_list.end(); it++) {
+        if ((*it) == client) {
+            SlitClients::iterator prev = it;
+            prev--;
+            iter_swap(it, prev);
+            reconfigure();
+            break;
+        }
+    }
+}
+
+void Slit::clientDown(SlitClient* client) {
+    if (!client || m_client_list.size() < 2)
+        return;
+    
+    if (client == m_client_list.back()) {
+        cycleClientsDown();
+        return;
+    }
+    
+    SlitClients::reverse_iterator it = m_client_list.rbegin();
+    for(it++; it != m_client_list.rend(); it++) {
+        if ((*it) == client) {
+            SlitClients::reverse_iterator next = it;
+            next--;
+            iter_swap(it, next);
+            reconfigure();
+            break;
+        }
+    }
+}
+
 void Slit::cycleClientsUp() {
     if (m_client_list.size() < 2)
         return;
@@ -1041,18 +1082,7 @@ void Slit::exposeEvent(XExposeEvent &ev) {
     // we don't need to clear the entire window 
     // just the are that gets exposed
     frame.window.clearArea(ev.x, ev.y, ev.width, ev.height);
-    if (m_transp.get()) {
-        if ((int)m_transp->alpha() != *m_rc_alpha)
-            m_transp->setAlpha(*m_rc_alpha);
-
-        if (screen().rootPixmap() != m_transp->source())
-            m_transp->setSource(screen().rootPixmap(), screen().screenNumber());
-
-        m_transp->render(frame.window.x() + ev.x, frame.window.y() + ev.y,
-                         ev.x, ev.y,
-                         ev.width, ev.height);
-        
-    }
+    frame.window.updateTransparent(ev.x, ev.y, ev.width, ev.height);
 }
 
 void Slit::update(FbTk::Subject *subj) {
@@ -1061,19 +1091,7 @@ void Slit::update(FbTk::Subject *subj) {
 
 void Slit::clearWindow() {
     frame.window.clear();
-    if (m_transp.get()) {
-        if ((int)m_transp->alpha() != *m_rc_alpha)
-            m_transp->setAlpha(*m_rc_alpha);
-
-        if (screen().rootPixmap() != m_transp->source())
-            m_transp->setSource(screen().rootPixmap(), screen().screenNumber());
-
-        m_transp->render(frame.window.x(), frame.window.y(),
-                         0, 0,
-                         frame.window.width(), frame.window.height());
-        
-    }
-
+    frame.window.updateTransparent();
 }
 
 void Slit::toggleHidden() {
@@ -1129,28 +1147,34 @@ void Slit::loadClientList(const char *filename) {
 void Slit::updateClientmenu() {
     if (screen().isShuttingdown()) 
         return;
+    _FB_USES_NLS;
 
     // clear old items
     m_clientlist_menu.removeAll();
-    m_clientlist_menu.setLabel("Clients");
+    m_clientlist_menu.setLabel(_FBTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"));
 
     FbTk::RefCount<FbTk::Command> cycle_up(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsUp));
     FbTk::RefCount<FbTk::Command> cycle_down(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsDown));
-    m_clientlist_menu.insert("Cycle Up", cycle_up);
-    m_clientlist_menu.insert("Cycle Down", cycle_down);
+    m_clientlist_menu.insert(_FBTEXT(Slit, CycleUp, "Cycle Up", "Cycle clients upwards"), cycle_up);
+    m_clientlist_menu.insert(_FBTEXT(Slit, CycleDown, "Cycle Down", "Cycle clients downwards"), cycle_down);
 
-    FbTk::MenuItem *separator = new FbTk::MenuItem("---");
-    separator->setEnabled(false);
-    m_clientlist_menu.insert(separator);
+    m_clientlist_menu.insert(new FbTk::MenuSeparator());
 
     FbTk::RefCount<FbTk::Command> reconfig(new FbTk::SimpleCommand<Slit>(*this, &Slit::reconfigure));
     SlitClients::iterator it = m_client_list.begin();
     for (; it != m_client_list.end(); ++it) {
         if ((*it) != 0 && (*it)->window() != 0)
-            m_clientlist_menu.insert(new SlitClientMenuItem(*(*it), reconfig));
+            m_clientlist_menu.insert(new SlitClientMenuItem(*this, *(*it), reconfig));
     }
 
-    m_clientlist_menu.update();
+    m_clientlist_menu.insert(new FbTk::MenuSeparator());
+    FbTk::RefCount<FbTk::Command> savecmd(new FbTk::SimpleCommand<Slit>(*this, &Slit::saveClientList));
+    m_clientlist_menu.insert(_FBTEXT(Slit, 
+                                     SaveSlitList,
+                                     "Save SlitList", "Saves the current order in the slit"), 
+                             savecmd);
+
+    m_clientlist_menu.updateMenu();
 }
 
 void Slit::saveClientList() {
@@ -1181,8 +1205,7 @@ void Slit::setAutoHide(bool val) {
 }
 
 void Slit::setupMenu() {
-    I18n *i18n = I18n::instance();
-    using namespace FBNLS;
+    _FB_USES_NLS;
     using namespace FbTk;
 
     FbTk::MacroCommand *s_a_reconf_macro = new FbTk::MacroCommand();
@@ -1201,97 +1224,96 @@ void Slit::setupMenu() {
     FbTk::RefCount<FbTk::Command> save_and_reconfigure_slit(s_a_reconf_slit_macro);
 
     // setup base menu
-    m_slitmenu.setLabel("Slit");
-    m_slitmenu.insert(i18n->getMessage(CommonSet, CommonPlacementTitle,
-                                       "Placement"),
+    m_slitmenu.setLabel(_FBTEXT(Slit, Slit, "Slit", "The Slit"));
+    m_slitmenu.insert(_FBTEXT(Menu, Placement, "Placement", "Title of Placement menu"),
                       &m_placement_menu);
 
-    m_slitmenu.insert("Layer...", m_layermenu.get());
+    m_slitmenu.insert(_FBTEXT(Menu, Layer, "Layer...", "Title of Layer menu"), m_layermenu.get());
 
 #ifdef XINERAMA
     if (screen().hasXinerama()) {
-        // TODO: nls (main label, plus menu heading)
-        m_slitmenu.insert("On Head...", new XineramaHeadMenu<Slit>(
-                                                                   screen().menuTheme(),
-                                                                   screen(),
-                                                                   screen().imageControl(),
-                                                                   *screen().layerManager().getLayer(Fluxbox::instance()->getMenuLayer()),
-                                                                   *this,
-                                                                   "Slit on Head"
-                                                                   ));
+        m_slitmenu.insert(_FBTEXT(Menu, OnHead, "On Head...", "Title of On Head menu"),
+                          new XineramaHeadMenu<Slit>(
+                              screen().menuTheme(),
+                              screen(),
+                              screen().imageControl(),
+                              *screen().layerManager().getLayer(Fluxbox::instance()->getMenuLayer()),
+                              *this,
+                              _FBTEXT(Slit, OnHead, "Slit on Head", "Title of Slits On Head menu")
+                              ));
     }
                     
 #endif //XINERAMA
-    m_slitmenu.insert(new BoolMenuItem(i18n->getMessage(CommonSet, CommonAutoHide,
-                                                        "Auto hide"),
+    m_slitmenu.insert(new BoolMenuItem(_FBTEXT(Common, AutoHide, "Auto hide", "This thing automatically hides when not close by"),
                                        *m_rc_auto_hide,
                                        save_and_reconfigure_slit));
 
-    m_slitmenu.insert(new BoolMenuItem("Maximize Over", 
+    m_slitmenu.insert(new BoolMenuItem(_FBTEXT(Common, MaximizeOver,"Maximize Over", "Maximize over this thing when maximizing"), 
                                        *m_rc_maximize_over, 
                                        save_and_reconfigure_slit));
 
     // this saves resources and clears the slit window to update alpha value
-    FbTk::MenuItem *alpha_menuitem = new IntResMenuItem("Alpha", 
-                                                        m_rc_alpha,
-                                                        0, 255);
+    FbTk::MenuItem *alpha_menuitem = 
+        new IntResMenuItem(_FBTEXT(Common, Alpha, "Alpha", "Transparency level"),
+                           m_rc_alpha,
+                           0, 255);
     // setup command for alpha value
     MacroCommand *alpha_macrocmd = new MacroCommand(); 
-    RefCount<Command> clear_cmd(new SimpleCommand<Slit>(*this, &Slit::clearWindow));
+    RefCount<Command> alpha_cmd(new SimpleCommand<Slit>(*this, &Slit::updateAlpha));
     alpha_macrocmd->add(saverc_cmd);
-    alpha_macrocmd->add(clear_cmd);
+    alpha_macrocmd->add(alpha_cmd);
     RefCount<Command> set_alpha_cmd(alpha_macrocmd);
     alpha_menuitem->setCommand(set_alpha_cmd);
 
     m_slitmenu.insert(alpha_menuitem);
 
-    m_slitmenu.insert(new SlitDirMenuItem(i18n->getMessage(SlitSet, SlitSlitDirection,
-                                                           "Slit Direction"), 
+    m_slitmenu.insert(new SlitDirMenuItem(_FBTEXT(Slit, Direction, "Slit Direction", "Orientation of slit"), 
                                           *this,
                                           save_and_reconfigure));
-    m_slitmenu.insert("Clients", &m_clientlist_menu);
-    m_slitmenu.update();
+    m_slitmenu.insert(_FBTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"), &m_clientlist_menu);
+    m_slitmenu.updateMenu();
 
     // setup sub menu
-    m_placement_menu.setLabel(i18n->getMessage(SlitSet, SlitSlitPlacement,
-                                               "Slit Placement"));
+    m_placement_menu.setLabel(_FBTEXT(Slit, Placement, "Slit Placement", "Slit Placement"));
     m_placement_menu.setMinimumSublevels(3);
     m_layermenu->setInternalMenu();
     m_clientlist_menu.setInternalMenu();
    
+    typedef pair<const char *, Slit::Placement> PlacementP;
+    typedef list<PlacementP> Placements;
+    Placements place_menu;
+
+    // menu is 3 wide, 5 down
+    place_menu.push_back(PlacementP(_FBTEXT(Align, TopLeft, "Top Left", "Top Left"), Slit::TOPLEFT));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, LeftCenter, "Left Center", "Left Center"), Slit::LEFTCENTER));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, BottomLeft, "Bottom Left", "Bottom Left"), Slit::BOTTOMLEFT));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, TopCenter, "Top Center", "Top Center"), Slit::TOPCENTER));
+    place_menu.push_back(PlacementP((const char *)0, Slit::TOPLEFT));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, BottomCenter, "Bottom Center", "Bottom Center"), Slit::BOTTOMCENTER));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, TopRight, "Top Right", "Top Right"), Slit::TOPRIGHT));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, RightCenter, "Right Center", "Right Center"), Slit::RIGHTCENTER));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, BottomRight, "Bottom Right", "Bottom Right"), Slit::BOTTOMRIGHT));
+    
 
-    // setup items in sub menu
-    struct {
-        int set;
-        int base;
-        const char *default_str;
-        Placement slit_placement;
-    } place_menu[]  = {
-        {CommonSet, CommonPlacementTopLeft, "Top Left", Slit::TOPLEFT},
-        {CommonSet, CommonPlacementCenterLeft, "Center Left", Slit::CENTERLEFT},
-        {CommonSet, CommonPlacementBottomLeft, "Bottom Left", Slit::BOTTOMLEFT},
-        {CommonSet, CommonPlacementTopCenter, "Top Center", Slit::TOPCENTER},
-        {0, 0, 0, Slit::TOPLEFT}, // middle item, empty
-        {CommonSet, CommonPlacementBottomCenter, "Bottom Center", Slit::BOTTOMCENTER},
-        {CommonSet, CommonPlacementTopRight, "Top Right", Slit::TOPRIGHT},
-        {CommonSet, CommonPlacementCenterRight, "Center Right", Slit::CENTERRIGHT},
-        {CommonSet, CommonPlacementBottomRight, "Bottom Right", Slit::BOTTOMRIGHT}
-    };
     // create items in sub menu
     for (size_t i=0; i<9; ++i) {
-        if (place_menu[i].default_str == 0) {
+        const char *str = place_menu.front().first;
+        Slit::Placement placement = place_menu.front().second;
+
+        if (str == 0) {
             m_placement_menu.insert("");
+            m_placement_menu.setItemEnabled(i, false);
         } else {
-            const char *i18n_str = i18n->getMessage(place_menu[i].set, 
-                                                    place_menu[i].base,
-                                                    place_menu[i].default_str);
-            m_placement_menu.insert(new PlaceSlitMenuItem(i18n_str, *this,
-                                                          place_menu[i].slit_placement,
+            m_placement_menu.insert(new PlaceSlitMenuItem(str, *this,
+                                                          placement, 
                                                           save_and_reconfigure));
+                                                              
         }
+        place_menu.pop_front();
     }
+
     // finaly update sub menu
-    m_placement_menu.update();
+    m_placement_menu.updateMenu();
 }
 
 void Slit::moveToLayer(int layernum) {
@@ -1305,3 +1327,12 @@ void Slit::saveOnHead(int head) {
     reconfigure();
 }
 
+void Slit::updateAlpha() {
+    // called when the alpha resource is changed
+    if (FbTk::Transparent::haveComposite()) {
+        frame.window.setOpaque(*m_rc_alpha);
+    } else {
+        frame.window.setAlpha(*m_rc_alpha);
+        clearWindow();
+    }
+}
