@@ -1,5 +1,5 @@
 // WinClient.cc for Fluxbox - an X11 Window manager
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen(at)users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen(at)users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,22 +19,27 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: WinClient.cc,v 1.38 2004/04/01 14:06:42 rathnor Exp $
+// $Id: WinClient.cc 3834 2005-01-14 09:35:41Z mathias $
 
 #include "WinClient.hh"
 
 #include "Window.hh"
 #include "fluxbox.hh"
 #include "Screen.hh"
-#include "I18n.hh"
 #include "FbAtoms.hh"
 #include "EventManager.hh"
 #include "Xutil.hh"
 
+#include "FbTk/I18n.hh"
+
 #include <iostream>
 #include <algorithm>
 #include <iterator>
-#include <cassert>
+#ifdef HAVE_CASSERT
+  #include <cassert>
+#else
+  #include <assert.h>
+#endif
 
 using namespace std;
 
@@ -147,10 +152,19 @@ void WinClient::updateRect(int x, int y,
 
 }
 
+bool WinClient::acceptsFocus() const {
+    return (m_focus_mode == F_LOCALLYACTIVE || 
+            m_focus_mode == F_PASSIVE || 
+            m_focus_mode == F_GLOBALLYACTIVE && send_focus_message);
+}
+
 bool WinClient::sendFocus() {
     if (!send_focus_message)
         return false;
-
+#ifdef DEBUG
+    cerr<<"WinClient::"<<__FUNCTION__<<": this = "<<this<<
+        " window = 0x"<<hex<<window()<<dec<<endl;
+#endif // DEBUG
     Display *disp = FbTk::App::instance()->display();
     // setup focus msg
     XEvent ce;
@@ -192,10 +206,6 @@ void WinClient::sendClose(bool forceful) {
     }
 }
 
-void WinClient::reparent(Window win, int x, int y) {
-    XReparentWindow(FbTk::App::instance()->display(), window(), win, x, y);
-}
-
 bool WinClient::getAttrib(XWindowAttributes &attr) const {
     return XGetWindowAttributes(FbTk::App::instance()->display(), window(), &attr);
 }
@@ -241,8 +251,12 @@ void WinClient::updateWMClassHint() {
 }
 
 void WinClient::updateTransientInfo() {
+#ifdef DEBUG
+    cerr<<__FUNCTION__<<": m_win = "<<m_win<<endl;
+#endif // DEBUG
     if (m_win == 0)
         return;
+
     // remove us from parent
     if (transientFor() != 0) {
         transientFor()->transientList().remove(this);
@@ -251,13 +265,21 @@ void WinClient::updateTransientInfo() {
     transient_for = 0;
     Display *disp = FbTk::App::instance()->display();
     // determine if this is a transient window
-    Window win;
-    if (!XGetTransientForHint(disp, window(), &win))
+    Window win = 0;
+    if (!XGetTransientForHint(disp, window(), &win)) {
+#ifdef DEBUG
+        cerr<<__FUNCTION__<<": window() = 0x"<<hex<<window()<<dec<<"Failed to read transient for hint."<<endl;
+#endif // DEBUG
         return;
+    }
 
     // we can't be transient to ourself
-    if (win == window())
+    if (win == window()) {
+#ifdef DEBUG
+        cerr<<__FUNCTION__<<": transient to ourself"<<endl;
+#endif // DEBUG
         return;
+    }
 	
     if (win != None && m_win->screen().rootWindow() == win) {
         // transient for root window... =  transient for group
@@ -268,6 +290,10 @@ void WinClient::updateTransientInfo() {
 
     transient_for = Fluxbox::instance()->searchWindow(win);
 
+#ifdef DEBUG
+    cerr<<__FUNCTION__<<": transient_for window = 0x"<<hex<<win<<dec<<endl;
+    cerr<<__FUNCTION__<<": transient_for = "<<transient_for<<endl;
+#endif // DEBUG
     // make sure we don't have deadlock loop in transient chain
     for (WinClient *w = this; w != 0; w = w->transient_for) {
         if (w == w->transient_for) {
@@ -288,7 +314,17 @@ void WinClient::updateTransientInfo() {
 
 
 void WinClient::updateTitle() {
-    m_title = Xutil::getWMName(window());
+    // why 512? very very long wmnames seem to either 
+    // crash fluxbox or to make it have high cpuload
+    // see also: 
+    //    http://www.securityfocus.com/archive/1/382398/2004-11-24/2004-11-30/2
+    // 
+    // TODO: - find out why this mostly happens when using xft-fonts
+    //       - why other windowmanagers (pekwm/pwm3/openbox etc) are
+    //         also influenced
+    //
+    // the limitation to 512 chars only avoids running in that trap
+    m_title = string(Xutil::getWMName(window()) ,0 , 512);
 }
 
 void WinClient::updateIconTitle() {
@@ -624,8 +660,10 @@ void WinClient::updateWMProtocols() {
         XFree(proto);
         if (m_win)
             m_win->updateFunctions();
+#ifdef DEBUG
     } else {
         cerr<<"Warning: Failed to read WM Protocols. "<<endl;
+#endif // DEBUG
     }
 
 }
