@@ -22,17 +22,17 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Workspace.cc,v 1.95 2004/03/21 09:00:25 rathnor Exp $
+// $Id: Workspace.cc 3770 2004-12-13 14:03:17Z mathias $
 
 #include "Workspace.hh"
 
-#include "I18n.hh"
 #include "fluxbox.hh"
 #include "Screen.hh"
 #include "Window.hh"
 #include "WinClient.hh"
 #include "FbWinFrame.hh"
 
+#include "FbTk/I18n.hh"
 #include "FbTk/MenuItem.hh"
 #include "FbTk/StringUtil.hh"
 
@@ -48,8 +48,16 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
-#include <cstdio>
-#include <cstring>
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
+#else
+  #include <stdio.h>
+#endif
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
 
 #include <algorithm>
 #include <iostream>
@@ -218,12 +226,12 @@ void Workspace::showAll() {
 }
 
 
-void Workspace::hideAll() {
+void Workspace::hideAll(bool interrupt_moving) {
     Windows::reverse_iterator it = m_windowlist.rbegin();
     Windows::reverse_iterator it_end = m_windowlist.rend();
     for (; it != it_end; ++it) {
         if (! (*it)->isStuck())
-            (*it)->withdraw();
+            (*it)->withdraw(interrupt_moving);
     }
 }
 
@@ -281,7 +289,7 @@ bool Workspace::checkGrouping(FluxboxWindow &win) {
 
     string instance_name = win.winClient().getWMClassName();
 
-    // go throu every group and search for matching win instancename
+    // go through every group and search for matching win instancename
     GroupList::iterator g(m_groups.begin());
     GroupList::iterator g_end(m_groups.end());
     for (; g != g_end; ++g) {
@@ -303,7 +311,8 @@ bool Workspace::checkGrouping(FluxboxWindow &win) {
                             (*g).end(), 
                             FindInGroup(*(*wit))) != (*g).end()) {
                     // make sure the window is groupable
-                    if ( !(*wit)->isGroupable() && (*wit)->winClient().fbwindow() == &win)
+                    // and don't group with ourself
+                    if ( !(*wit)->isGroupable() || (*wit)->winClient().fbwindow() == &win)
                         break; // try next name
 #ifdef DEBUG
                     cerr<<__FILE__<<"("<<__FUNCTION__<<"): window ("<<*wit<<") attaching window ("<<&win<<")"<<endl;
@@ -344,7 +353,7 @@ bool Workspace::loadGroups(const std::string &filename) {
 }
 
 void Workspace::update(FbTk::Subject *subj) {
-    menu().update();
+    menu().updateMenu();
 }
 
 
@@ -352,18 +361,19 @@ void Workspace::setName(const std::string &name) {
     if (!name.empty()) {
         m_name = name;
     } else { //if name == 0 then set default name from nls
+        _FB_USES_NLS;
         char tname[128];
-        sprintf(tname, I18n::instance()->
-                getMessage(FBNLS::WorkspaceSet, 
-                           FBNLS::WorkspaceDefaultNameFormat,
-                           "Workspace %d"), m_id + 1); //m_id starts at 0
+        sprintf(tname, 
+                _FBTEXT(Workspace, DefaultNameFormat, 
+                        "Workspace %d", "Default workspace names, with a %d for the workspace number"),
+                m_id + 1); //m_id starts at 0
         m_name = tname;
     }
 	
     screen().updateWorkspaceNamesAtom();
 	
     menu().setLabel(m_name.c_str());
-    menu().update();
+    menu().updateMenu();
 }
 
 /**
@@ -397,7 +407,7 @@ void Workspace::updateClientmenu() {
             menu().insert(new ClientMenuItem(*(*client_it)));
     }
 
-    menu().update();
+    menu().updateMenu();
 }
 
 void Workspace::placeWindow(FluxboxWindow &win) {
@@ -445,14 +455,14 @@ void Workspace::placeWindow(FluxboxWindow &win) {
         if (test_x < head_left)
             test_x = head_left;
 
-        if (test_x > head_right)
-            test_x = head_right;
+        if (test_x + win_w > head_right)
+            test_x = head_right - win_w;
 
         if (test_y < head_top)
             test_y = head_top;
 
-        if (test_y > head_bot)
-            test_y = head_bot;
+        if (test_y + win_h > head_bot)
+            test_y = head_bot - win_h;
 
         place_x = test_x;
         place_y = test_y;
