@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: WorkspaceCmd.cc,v 1.10 2004/01/16 09:21:31 fluxgen Exp $
+// $Id: WorkspaceCmd.cc 3700 2004-11-19 11:37:27Z mathias $
 
 #include "WorkspaceCmd.hh"
 
@@ -28,10 +28,15 @@
 #include "Window.hh"
 #include "Screen.hh"
 #include "fluxbox.hh"
+#include "WinClient.hh"
 
 #include "FbTk/KeyUtil.hh"
 
-#include <cmath>
+#ifdef HAVE_CMATH
+  #include <cmath>
+#else
+  #include <math.h>
+#endif
 #include <algorithm>
 #include <functional>
 #include <iostream>
@@ -49,7 +54,8 @@ void NextWindowCmd::execute() {
             else {
                 // if stacked cycling, then set a watch for 
                 // the release of exactly these modifiers
-                if (!fb->watchingScreen() && !(m_option & BScreen::CYCLELINEAR))
+                if (!fb->watchingScreen() && 
+                    !(m_option & BScreen::CYCLELINEAR))
                     Fluxbox::instance()->watchKeyRelease(*screen, mods);
                 screen->nextFocus(m_option);
             }
@@ -70,7 +76,8 @@ void PrevWindowCmd::execute() {
             else {
                 // if stacked cycling, then set a watch for 
                 // the release of exactly these modifiers
-                if (!fb->watchingScreen() && !(m_option & BScreen::CYCLELINEAR))
+                if (!fb->watchingScreen() 
+                    && !(m_option & BScreen::CYCLELINEAR))
                     Fluxbox::instance()->watchKeyRelease(*screen, mods);
                 screen->prevFocus(m_option);
             }
@@ -79,6 +86,18 @@ void PrevWindowCmd::execute() {
     }
 }
 
+void DirFocusCmd::execute() {
+    BScreen *screen = Fluxbox::instance()->keyScreen();
+    if (screen == 0)
+        return;
+
+    WinClient *client = Fluxbox::instance()->getFocusedWindow();
+    if (client == 0 || client->fbwindow() == 0)
+        return;
+
+    screen->dirFocus(*client->fbwindow(), m_dir);
+}
+
 void NextWorkspaceCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
     if (screen != 0)
@@ -128,7 +147,7 @@ void ArrangeWindowsCmd::execute() {
     const unsigned int max_heigth = screen->maxBottom(head) - screen->maxTop(head);
 
 	// try to get the same number of rows as columns.
-	unsigned int rows = int(sqrt(win_count));  // truncate to lower
+	unsigned int rows = int(sqrt((float)win_count));  // truncate to lower
 	unsigned int cols = int(0.99 + float(win_count) / float(rows));
 	if (max_width<max_heigth) {	// rotate
 		unsigned int tmp;
