@@ -1,5 +1,5 @@
 // fluxbox.cc for Fluxbox Window Manager
-// Copyright (c) 2001 - 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at users.sourceforge.net)
 //
 // blackbox.cc for blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: fluxbox.cc 3720 2004-11-24 12:26:12Z mathias $
+// $Id: fluxbox.cc 3835 2005-01-14 09:40:40Z mathias $
 
 #include "fluxbox.hh"
 
@@ -38,6 +38,7 @@
 
 #include "FbTk/I18n.hh"
 #include "FbTk/Image.hh"
+#include "FbTk/FileUtil.hh"
 #include "FbTk/KeyUtil.hh"
 #include "FbTk/ImageControl.hh"
 #include "FbTk/EventManager.hh"
@@ -151,24 +152,11 @@ class Toolbar { };
 using namespace std;
 using namespace FbTk;
 
-static Window last_bad_window = None;
 namespace {
-void copyFile(const std::string &from, const std::string &to) {
-        ifstream from_file(from.c_str());
-        ofstream to_file(to.c_str());
 
-        if (! to_file.good()) {
-            cerr<<"Can't write file: "<<to<<endl;
-        } else if (from_file.good()) {
-            to_file<<from_file.rdbuf(); //copy file
-        } else {
-            cerr<<"Can't copy from "<<from<<" to "<<to<<endl;
-        }
-}
-
-} // end anonymous
-
-static int handleXErrors(Display *d, XErrorEvent *e) {
+Window last_bad_window = None;
+ 
+int handleXErrors(Display *d, XErrorEvent *e) {
     if (e->error_code == BadWindow)
         last_bad_window = e->resourceid;
 #ifdef DEBUG
@@ -183,10 +171,10 @@ static int handleXErrors(Display *d, XErrorEvent *e) {
     }
 #endif // !DEBUG
 
-
     return False;
 }
 
+} // end anonymous
 
 //static singleton var
 Fluxbox *Fluxbox::s_singleton=0;
@@ -213,11 +201,11 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
       m_rc_numlayers(m_resourcemanager, 13, "session.numLayers", "Session.NumLayers"),
       m_rc_double_click_interval(m_resourcemanager, 250, "session.doubleClickInterval", "Session.DoubleClickInterval"),
       m_rc_update_delay_time(m_resourcemanager, 0, "session.updateDelayTime", "Session.UpdateDelayTime"),
-      m_rc_stylefile(m_resourcemanager, "", "session.styleFile", "Session.StyleFile"),
+      m_rc_stylefile(m_resourcemanager, DEFAULTSTYLE, "session.styleFile", "Session.StyleFile"),
       m_rc_menufile(m_resourcemanager, DEFAULTMENU, "session.menuFile", "Session.MenuFile"),
       m_rc_keyfile(m_resourcemanager, DEFAULTKEYSFILE, "session.keyFile", "Session.KeyFile"),
-      m_rc_slitlistfile(m_resourcemanager, "", "session.slitlistFile", "Session.SlitlistFile"),
-      m_rc_groupfile(m_resourcemanager, "", "session.groupFile", "Session.GroupFile"),
+      m_rc_slitlistfile(m_resourcemanager, "~/.fluxbox/slitlist", "session.slitlistFile", "Session.SlitlistFile"),
+      m_rc_groupfile(m_resourcemanager, "~/.fluxbox/groups", "session.groupFile", "Session.GroupFile"),
       m_rc_appsfile(m_resourcemanager, "~/.fluxbox/apps", "session.appsFile", "Session.AppsFile"),
       m_rc_titlebar_left(m_resourcemanager, 
                          TitlebarList(&s_titlebar_left[0], &s_titlebar_left[1]), 
@@ -256,8 +244,6 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
                              "Error message when no X display appears to exist"));
     }
 
-    FbTk::fontInit();
-
     Display *disp = FbTk::App::instance()->display();
     // For KDE dock applets
     // KDE v1.x
@@ -573,15 +559,15 @@ void Fluxbox::setupConfigFiles() {
 
     // copy key configuration
     if (create_keys)
-        copyFile(DEFAULTKEYSFILE, keys_file);
+        FbTk::FileUtil::copyFile(DEFAULTKEYSFILE, keys_file.c_str());
 
     // copy menu configuration
     if (create_menu)
-        copyFile(DEFAULTMENU, menu_file);
+        FbTk::FileUtil::copyFile(DEFAULTMENU, menu_file.c_str());
 
     // copy init file
     if (create_init)
-        copyFile(DEFAULT_INITFILE, init_file);
+        FbTk::FileUtil::copyFile(DEFAULT_INITFILE, init_file.c_str());
 
 }
 
@@ -914,14 +900,14 @@ void Fluxbox::handleButtonEvent(XButtonEvent &be) {
 
 void Fluxbox::handleUnmapNotify(XUnmapEvent &ue) {
 
+    BScreen *screen = searchScreen(ue.event);
+    
+    if (ue.event != ue.window && (!screen || !ue.send_event)) {
+        return;
+    }
 
     WinClient *winclient = 0;
 
-    BScreen *screen = searchScreen(ue.event);
-
-    if ( ue.event != ue.window && (screen != 0 || !ue.send_event))
-        return;
-
     if ((winclient = searchWindow(ue.window)) != 0) {
 
         if (winclient != 0) {
@@ -943,6 +929,14 @@ void Fluxbox::handleUnmapNotify(XUnmapEvent &ue) {
                 win = 0;
             }
         }
+
+    // according to http://tronche.com/gui/x/icccm/sec-4.html#s-4.1.4
+    // a XWithdrawWindow is 
+    //   1) unmapping the window (which leads to the upper branch
+    //   2) sends an synthetic unampevent (which is handled below)
+    } else if (screen && ue.send_event) {
+        XDeleteProperty(display(), ue.window, FbAtoms::instance()->getWMStateAtom());
+        XUngrabButton(display(), AnyButton, AnyModifier, ue.window);
     }
 
 }
@@ -969,19 +963,13 @@ void Fluxbox::handleClientMessage(XClientMessageEvent &ce) {
 
     if (ce.message_type == m_fbatoms->getWMChangeStateAtom()) {
         WinClient *winclient = searchWindow(ce.window);
-        if (! winclient || !winclient->fbwindow() || ! winclient->validateClient()) {
-            cerr<<"Couldn't find window! for WM_CHANGE_STATE"<<endl;
+        if (! winclient || !winclient->fbwindow() || ! winclient->validateClient())
             return;
-        }
 
-        if (ce.data.l[0] == IconicState) {
-            cerr<<"WM_CHANGE_STATE is IconicState"<<endl;
+        if (ce.data.l[0] == IconicState)
             winclient->fbwindow()->iconify();
-        }
-        if (ce.data.l[0] == NormalState) {
-            cerr<<"WM_CHANGE_STATE is NormalState"<<endl;
+        if (ce.data.l[0] == NormalState)
             winclient->fbwindow()->deiconify();
-        }
     } else if (ce.message_type == m_fbatoms->getFluxboxChangeWorkspaceAtom()) {
         BScreen *screen = searchScreen(ce.window);
 
@@ -1387,7 +1375,6 @@ void Fluxbox::shutdown() {
              m_screen_list.end(), mem_fun(&BScreen::shutdown));
 
     sync(false);
-
 }
 
 /// saves resources
@@ -1502,7 +1489,9 @@ void Fluxbox::load_rc() {
         *m_rc_stylefile = DEFAULTSTYLE;
 
     if (!Workspace::loadGroups(*m_rc_groupfile)) {
+#ifdef DEBUG
         cerr<<_FBTEXT(Fluxbox, CantLoadGroupFile, "Failed to load groupfile", "Couldn't load the groupfile")<<": "<<*m_rc_groupfile<<endl;
+#endif // DEBUG
     }
 }
 
@@ -1650,10 +1639,11 @@ bool Fluxbox::menuTimestampsChanged() const {
     std::list<MenuTimestamp *>::const_iterator it = m_menu_timestamps.begin();
     std::list<MenuTimestamp *>::const_iterator it_end = m_menu_timestamps.end();
     for (; it != it_end; ++it) {
-        struct stat buf;
+        
+        time_t timestamp = FbTk::FileUtil::getLastStatusChangeTimestamp((*it)->filename.c_str());
 
-        if (! stat((*it)->filename.c_str(), &buf)) {
-            if ((*it)->timestamp != buf.st_ctime)
+        if (timestamp >= 0) {
+            if (timestamp != (*it)->timestamp)
                 return true;
         } else
             return true;
@@ -1689,13 +1679,12 @@ void Fluxbox::rereadMenu(bool show_after_reread) {
 
 
 void Fluxbox::real_rereadMenu() {
-    std::list<MenuTimestamp *>::iterator it = m_menu_timestamps.begin();
-    std::list<MenuTimestamp *>::iterator it_end = m_menu_timestamps.end();
-    for (; it != it_end; ++it)
-        delete *it;
-
-    m_menu_timestamps.erase(m_menu_timestamps.begin(), m_menu_timestamps.end());
-    for_each(m_screen_list.begin(), m_screen_list.end(), mem_fun(&BScreen::rereadMenu));
+    
+    clearMenuFilenames();
+    
+    for_each(m_screen_list.begin(), 
+             m_screen_list.end(), 
+             mem_fun(&BScreen::rereadMenu));
 
     if(m_show_menu_after_reread) {
 
@@ -1722,13 +1711,13 @@ void Fluxbox::saveMenuFilename(const char *filename) {
     }
 
     if (! found) {
-        struct stat buf;
+        time_t timestamp = FbTk::FileUtil::getLastStatusChangeTimestamp(filename);
 
-        if (! stat(filename, &buf)) {
+        if (timestamp >= 0) {
             MenuTimestamp *ts = new MenuTimestamp;
 
             ts->filename = filename;
-            ts->timestamp = buf.st_ctime;
+            ts->timestamp = timestamp;
 
             m_menu_timestamps.push_back(ts);
         }
@@ -1736,12 +1725,10 @@ void Fluxbox::saveMenuFilename(const char *filename) {
 }
 
 void Fluxbox::clearMenuFilenames() {
-    std::list<MenuTimestamp *>::iterator it = m_menu_timestamps.begin();
-    std::list<MenuTimestamp *>::iterator it_end = m_menu_timestamps.end();
-    for (; it != it_end; ++it)
-         delete *it;
-
-    m_menu_timestamps.erase(m_menu_timestamps.begin(), m_menu_timestamps.end());
+    while(!m_menu_timestamps.empty()) {
+        delete m_menu_timestamps.back();
+        m_menu_timestamps.pop_back();
+    }
 }
 
 void Fluxbox::timed_reconfigure() {
