@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbRun.cc,v 1.30 2004/04/22 21:01:58 fluxgen Exp $
+// $Id: FbRun.cc 3795 2004-12-24 03:02:48Z mathias $
 
 #include "FbRun.hh"
 
@@ -27,7 +27,7 @@
 #include "EventManager.hh"
 #include "Color.hh"
 #include "KeyUtil.hh"
-#include "Directory.hh"
+#include "FileUtil.hh"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -48,7 +48,11 @@
 #include <iterator>
 #include <fstream>
 #include <algorithm>
-#include <cassert>
+#ifdef HAVE_CASSERT
+  #include <cassert>
+#else
+  #include <assert.h>
+#endif
 
 using namespace std;
 FbRun::FbRun(int x, int y, size_t width):
@@ -136,20 +140,27 @@ void FbRun::run(const std::string &command) {
             }
         }
 
-        // now m_current_history_item points at the duplicate, or
-        // at m_history.size() if no duplicate
         fstream inoutfile(m_history_file.c_str(), ios::in|ios::out);
         if (inoutfile) {
-            int i = 0;
-            // read past history items before current
-            for (string line; !inoutfile.eof() && i < m_current_history_item; i++)
-                getline(inoutfile, line);
-            // write the history items that come after current
-            for (i++; i < m_history.size(); i++)
-                inoutfile<<m_history[i]<<endl;
-            
-            // and append the current one back to the end
-            inoutfile<<text()<<endl;
+            // now m_current_history_item points at the duplicate, or
+            // at m_history.size() if no duplicate
+            if (m_current_history_item != m_history.size()) {
+                int i = 0;
+                // read past history items before current
+                for (; inoutfile.good() && i < m_current_history_item; i++)
+                    inoutfile.ignore(1, '\n');
+
+                // write the history items that come after current
+                for (i++; i < m_history.size(); i++)
+                    inoutfile<<m_history[i]<<endl;
+                
+            } else {
+                // set put-pointer at end of file
+                inoutfile.seekp(0, ios::end);
+            }
+            // append current command to the file
+            inoutfile<<command<<endl;
+
         } else
             cerr<<"FbRun Warning: Can't write command history to file: "<<m_history_file<<endl;
     }
@@ -162,7 +173,11 @@ bool FbRun::loadHistory(const char *filename) {
     ifstream infile(filename);
     if (!infile) {
         //even though we fail to load file, we should try save to it
-        m_history_file = filename;
+        ofstream outfile(filename);
+        if (outfile) {
+            m_history_file = filename;
+            return true;
+        }
         return false;
     }
     // clear old history and load new one from file
@@ -250,6 +265,7 @@ void FbRun::keyPressEvent(XKeyEvent &ke) {
             hide();
             FbTk::App::instance()->end(); // end program
             break;
+        case XK_KP_Enter:
         case XK_Return:
             run(text());
             break;
@@ -357,7 +373,8 @@ void FbRun::tabCompleteApps() {
         
         string path;
         
-        if(!prefix.empty() && prefix[0] =='/') {
+        if(!prefix.empty() && 
+            string("/.~").find_first_of(prefix[0]) != string::npos) {
             size_t rseparator= prefix.find_last_of("/");
             path= prefix.substr(0, rseparator + 1) +  ":";
             add_dirs= true;
@@ -383,18 +400,18 @@ void FbRun::tabCompleteApps() {
                                     filename;
 
                         // directories in dirmode ?
-                        if (add_dirs && dir.isDirectory(fncomplete) &&
+                        if (add_dirs && FbTk::FileUtil::isDirectory(fncomplete.c_str()) &&
                             filename != ".." && filename != ".") {
                             m_apps.push_back(fncomplete); 
                         // executables in dirmode ?
-                        } else if (add_dirs && dir.isRegularFile(fncomplete) && 
-                                   dir.isExecutable(fncomplete) && 
+                        } else if (add_dirs && FbTk::FileUtil::isRegularFile(fncomplete.c_str()) && 
+                                   FbTk::FileUtil::isExecutable(fncomplete.c_str()) && 
                                    (prefix == "" || 
                                     fncomplete.substr(0, prefix.size()) == prefix)) {
                             m_apps.push_back(fncomplete);
                         // executables in $PATH ?
-                        } else if (dir.isRegularFile(fncomplete) && 
-                                   dir.isExecutable(fncomplete) && 
+                        } else if (FbTk::FileUtil::isRegularFile(fncomplete.c_str()) && 
+                                   FbTk::FileUtil::isExecutable(fncomplete.c_str()) && 
                                    (prefix == "" || 
                                     filename.substr(0, prefix.size()) == prefix)) {
                             m_apps.push_back(filename);
@@ -430,7 +447,7 @@ void FbRun::tabCompleteApps() {
             }
             if (m_apps[apps_item].find(prefix) == 0) {
                 m_current_apps_item = apps_item;
-                if (FbTk::Directory::isDirectory(m_apps[m_current_apps_item]))
+                if (add_dirs && FbTk::FileUtil::isDirectory(m_apps[m_current_apps_item].c_str()))
                     setText(m_apps[m_current_apps_item] +  "/");
                 else
                     setText(m_apps[m_current_apps_item]);
